@@ -14,6 +14,142 @@
 
 #include "../ImportExport/ImportExportOsmBin.h"
 
+OsbRegion::OsbRegion(ImportExportOsmBin* osb) 
+	: region(0), device(0), isWorld(false), theOsb(osb)
+{
+}
+
+OsbRegion::~OsbRegion()
+{
+	if (isWorld) {
+		if (device)
+			device->close();
+		delete device;
+		delete theOsb;
+	}
+}
+
+bool OsbRegion::load(qint32 rg, MapDocument* d, OsbMapLayer* theLayer) 
+{
+	region = rg;
+
+	quint64 tocPos = theOsb->theRegionToc[rg];
+	if (tocPos) {
+		QDataStream ds(device);
+		device->seek(tocPos);
+		QList< QPair < qint32, quint64 > > aTileList;
+
+		ds >> aTileList;
+
+		for (int i=0; i<aTileList.size(); ++i) {
+			theTileToc[aTileList[i].first] = aTileList[i].second;
+		}
+		theRegionIndex = aTileList;
+
+		bool OK = true;
+		qint32 theRegionTile = theRegionIndex[0].first;
+		if (theRegionTile < 0)
+			OK = loadTile(theRegionTile, d, theLayer);
+
+		return OK;
+	} else {
+		isWorld = true;
+
+		theOsb = new ImportExportOsmBin(NULL);
+		QFileInfo fi( *((QFile*)device) );
+		QString p = fi.absolutePath();
+		device = NULL;
+		theOsb->loadFile(QString("%1/%2.osb").arg(p).arg(QString::number(rg)));
+		if (!theOsb->import(NULL))
+			return false;
+
+		device = new QFile(QString("%1/%2.osb").arg(p).arg(QString::number(rg)));
+		device->open(QIODevice::ReadOnly);
+		if (!device->isOpen()) {
+			SAFE_DELETE(device);
+			return false;
+		}
+
+		device->seek(theOsb->theRegionToc[rg]);
+		QList< QPair < qint32, quint64 > > aTileList;
+
+		QDataStream ds(device);
+		ds >> aTileList;
+
+		for (int i=0; i<aTileList.size(); ++i) {
+			theTileToc[aTileList[i].first] = aTileList[i].second;
+		}
+		theRegionIndex = aTileList;
+
+		bool OK = false;
+		qint32 theRegionTile = theRegionIndex[0].first;
+		if (theRegionTile < 0)
+			OK = loadTile(theRegionTile, d, theLayer);
+
+		return OK;
+	}
+}
+
+bool OsbRegion::loadTile(qint32 tile, MapDocument* d, OsbMapLayer* theLayer)
+{
+	if (!theTileToc.contains(tile))
+		return false;
+
+	QDataStream ds(device);
+	quint32 featCount;
+
+	quint32 pos = theTileToc[tile];
+	device->seek(pos);
+	ds >> featCount;
+
+	for (quint32 i=0; i<featCount; ++i) {
+		MapFeature* F = theOsb->getFeature(this, d, theLayer);
+		theOsb->theTileIndex[tile].push_back(F);
+		theLayer->featRefCount[F]++;
+	}
+
+	return true;
+}
+
+bool OsbRegion::clearRegion(MapDocument* d, OsbMapLayer* theLayer)
+{
+	bool OK = true;
+	qint32 theRegionTile = theRegionIndex[0].first;
+	if (theRegionTile < 0)
+		OK = clearTile(theRegionTile, d, theLayer);
+	return OK;
+}
+
+bool OsbRegion::clearTile(qint32 tile, MapDocument* d, OsbMapLayer* theLayer)
+{
+	Q_UNUSED(d);
+
+	QDataStream ds(device);
+	bool OK = true;
+
+	for (qint32 i=0; i<theOsb->theTileIndex[tile].size(); ++i) {
+		MapFeature* F = theOsb->theTileIndex[tile][i];
+		if (F && theLayer->exists(F) && theLayer->featRefCount.contains(F)) {
+			int theCount = --theLayer->featRefCount[F];
+			if (!(theCount)) {
+				quint32 j;
+				for (j=0; j<F->sizeParents(); ++j)
+					if (F->getParent(j)->layer() != theLayer)
+						break;
+				if (j == F->sizeParents()) {
+					theLayer->deleteFeature(F);
+					theLayer->featRefCount.remove(F);
+				}
+			}
+		} else {
+			theLayer->featRefCount.remove(F);
+		}
+	}
+	theOsb->theTileIndex.remove(tile);
+
+	return OK;
+}
+
 ImportExportOsmBin::ImportExportOsmBin(MapDocument* doc)
  : IImportExport(doc)
 {
@@ -62,10 +198,12 @@ void ImportExportOsmBin::addTileIndex(MapFeature* F)
 					(F->tagValue("waterway", "") == "river") ||
 					(F->tagValue("waterway", "") == "riverbank") ||
 					(F->tagValue("waterway", "") == "canal") ||
-					false)
+                    (F->tagValue("natural", "") == "coastline") ||
+                    //(F->tagValue("landuse", "__NULL__") != "__NULL__") ||
+                    false)
 				tile = -rg;
 
-			if (TrackPoint* N = CAST_NODE(F)) {
+			if (/*TrackPoint* N = */CAST_NODE(F)) {
 				if (!theTileNodesIndex[tile].contains(F))
 					theTileNodesIndex[tile].push_back(F);
 			}
@@ -76,7 +214,7 @@ void ImportExportOsmBin::addTileIndex(MapFeature* F)
 				if (!theTileRoadsIndex[tile].contains(F))
 					theTileRoadsIndex[tile].push_back(F);
 			}
-			if (Relation* L = CAST_RELATION(F)) {
+			if (/*Relation* L = */CAST_RELATION(F)) {
 				//for (unsigned int k=0; k<L->size(); ++k) {
 				//	if (TrackPoint* N2 = CAST_NODE(L->get(k)))
 				//		if (!theTileNodesIndex[j*NUM_TILES+i].contains(N2))
@@ -125,21 +263,23 @@ void ImportExportOsmBin::tagsFromBinary(MapFeature * F, QDataStream& ds)
 	for (unsigned int i=0; i < numTags; ++i) {
 		ds >> k;
 		ds >> v;
-		cur_pos = Device->pos();
-		if (keyTable.contains(k))
-			K = keyTable[k];
-		else {
-			Device->seek(k);
-			ds >> K;
+		if (F) {
+			cur_pos = ds.device()->pos();
+			if (keyTable.contains(k))
+				K = keyTable[k];
+			else {
+				ds.device()->seek(k);
+				ds >> K;
+			}
+			if (valueTable.contains(v))
+				V = valueTable[v];
+			else {
+				ds.device()->seek(v);
+				ds >> V;
+			}
+			F->setTag(K,V);
+			ds.device()->seek(cur_pos);
 		}
-		if (valueTable.contains(v))
-			V = valueTable[v];
-		else {
-			Device->seek(v);
-			ds >> V;
-		}
-		F->setTag(K,V);
-		Device->seek(cur_pos);
 	}
 }
 
@@ -184,26 +324,26 @@ bool ImportExportOsmBin::writeHeader(QDataStream& ds)
 	return true;
 }
 
-bool ImportExportOsmBin::writeIndex(QDataStream& ds)
+bool ImportExportOsmBin::writeIndex(QDataStream& ds, int selRegion)
 {
-	QMapIterator<qint32, QList<MapFeature*> > itN(theTileNodesIndex);
+	QMapIterator<qint32, QList<MapFeaturePtr> > itN(theTileNodesIndex);
 	while(itN.hasNext()) {
 		itN.next();
 		theTileIndex[itN.key()] += itN.value();
 	}
-	QMapIterator<qint32, QList<MapFeature*> > itR(theTileRoadsIndex);
+	QMapIterator<qint32, QList<MapFeaturePtr> > itR(theTileRoadsIndex);
 	while(itR.hasNext()) {
 		itR.next();
 		theTileIndex[itR.key()] += itR.value();
 	}
-	QMapIterator<qint32, QList<MapFeature*> > itL(theTileRelationsIndex);
+	QMapIterator<qint32, QList<MapFeaturePtr> > itL(theTileRelationsIndex);
 	while(itL.hasNext()) {
 		itL.next();
 		theTileIndex[itL.key()] += itL.value();
 	}
 
 	int x, y, rg;
-	QMapIterator< qint32, QList<MapFeature*> > it(theTileIndex);
+	QMapIterator< qint32, QList<MapFeaturePtr> > it(theTileIndex);
 	while (it.hasNext()) {
 		it.next();
 
@@ -215,23 +355,43 @@ bool ImportExportOsmBin::writeIndex(QDataStream& ds)
 			rg = (y * NUM_REGIONS / NUM_TILES) * NUM_REGIONS + (x * NUM_REGIONS / NUM_TILES);
 		}
 
-		theRegionIndex[rg].append(QPair < qint32, quint64 > (it.key(), Device->pos()));
-		writeFeatures(it.value(), ds);
+		if (rg == selRegion || selRegion == -1) {
+			theRegionIndex[rg].append(QPair < qint32, quint64 > (it.key(), Device->pos()));
+			writeFeatures(it.value(), ds);
+		}
 	}
 
 	QMapIterator < qint32, QList< QPair < qint32, quint64 > > > j(theRegionIndex);
 	while (j.hasNext()) {
 		j.next();
 
-		theRegionToc[j.key()] = Device->pos();
+		theRegionToc[j.key()] = ds.device()->pos();
 		ds << j.value();
 	}
-	tocPos = Device->pos();
+	tocPos = ds.device()->pos();
 	ds << theRegionToc;
 
 	return true;
 }
 
+bool ImportExportOsmBin::readWorld(QDataStream& ds)
+{
+	if (! readHeader(ds) ) return false;
+	if (! readRegionToc(ds) ) return false;
+
+	return true;
+}
+
+void ImportExportOsmBin::addWorldRegion(int region)
+{
+	theRegionToc[region] = 0;
+}
+
+void ImportExportOsmBin::removeWorldRegion(int region)
+{
+	theRegionToc.remove(region);
+}
+
 bool ImportExportOsmBin::writeTagLists(QDataStream& ds)
 {
 	QMultiMap <qint32, QString> popularityKey;
@@ -324,10 +484,10 @@ bool ImportExportOsmBin::writeTagLists(QDataStream& ds)
 //	return true;
 //}
 //
-bool ImportExportOsmBin::writeFeatures(QList<MapFeature*> theFeatList, QDataStream& ds)
+bool ImportExportOsmBin::writeFeatures(QList<MapFeaturePtr> theFeatList, QDataStream& ds)
 {
 	ds << (qint32)theFeatList.size();
-	QListIterator<MapFeature*> i(theFeatList);
+	QListIterator<MapFeaturePtr> i(theFeatList);
 	while (i.hasNext()) {
 		MapFeature* F = i.next();
 
@@ -357,7 +517,7 @@ bool ImportExportOsmBin::readHeader(QDataStream& ds)
 
 bool ImportExportOsmBin::readRegionToc(QDataStream& ds)
 {
-	Device->seek(tocPos);
+	ds.device()->seek(tocPos);
 	ds >> theRegionToc;
 
 	return true;
@@ -369,6 +529,9 @@ bool ImportExportOsmBin::readPopularTagLists(QDataStream& ds)
 	QString s;
 	quint64 cur_pos;
 	
+	if (!tagKeysPos)
+		return true;
+
 	Device->seek(tagKeysPos);
 	ds >> tagKeysSize;
 
@@ -443,109 +606,60 @@ bool ImportExportOsmBin::loadRegion(qint32 rg, MapDocument* d, OsbMapLayer* theL
 	if (!theRegionToc.contains(rg))
 		return false;
 
-	QDataStream ds(Device);
-	Device->seek(theRegionToc[rg]);
-	QList< QPair < qint32, quint64 > > aTileList;
-
-	ds >> aTileList;
-
-	for (int i=0; i<aTileList.size(); ++i) {
-		theTileToc[aTileList[i].first] = aTileList[i].second;
-	}
-	theRegionIndex[rg] = aTileList;
-
-	bool OK;
-	qint32 theRegionTile = theRegionIndex[rg][0].first;
-	if (theRegionTile < 0)
-		OK = loadTile(theRegionTile, d, theLayer);
-
-	return OK;
-}
-
-bool ImportExportOsmBin::loadTile(qint32 tile, MapDocument* d, OsbMapLayer* theLayer)
-{
-	QDataStream ds(Device);
-	quint32 featCount;
-
-	if (!theTileToc.contains(tile))
+	OsbRegion* org = new OsbRegion(this);
+	org->device = Device;
+	if (!org->load(rg, d, theLayer)) {
+		delete org;
 		return false;
-
-	quint32 pos = theTileToc[tile];
-	Device->seek(pos);
-	ds >> featCount;
-
-	for (quint32 i=0; i<featCount; ++i) {
-		MapFeature* F = getFeature(d, theLayer);
-		theTileIndex[tile].push_back(F);
-		featRefCount[F]++;
 	}
-
-	//qDebug() << "--------- " << tile;
-	//QMapIterator<MapFeature*, quint32> itN(featRefCount);
-	//while(itN.hasNext()) {
-	//	itN.next();
-
-	//	if (itN.value() > 1)
-	//		qDebug() << itN.key()->id() << " - " << itN.value();
-	//}
+	theRegionList[rg] = org;
 
 	return true;
 }
 
+bool ImportExportOsmBin::loadTile(qint32 tile, MapDocument* d, OsbMapLayer* theLayer)
+{
+	int y = int(tile / NUM_TILES);
+	int x = (tile % NUM_TILES);
+	int rg = (y * NUM_REGIONS / NUM_TILES) * NUM_REGIONS + (x * NUM_REGIONS / NUM_TILES);
+	if (!theRegionList.contains(rg))
+		return false;
+
+	return theRegionList[rg]->loadTile(tile, d, theLayer);
+}
+
 bool ImportExportOsmBin::clearRegion(qint32 rg, MapDocument* d, OsbMapLayer* theLayer)
 {
-	bool OK;
-	qint32 theRegionTile = theRegionIndex[rg][0].first;
-	if (theRegionTile < 0)
-		OK = clearTile(theRegionTile, d, theLayer);
-	theRegionIndex.remove(rg);
+	if (!theRegionList.contains(rg))
+		return false;
 
-	return OK;
+	theRegionList[rg]->clearRegion(d, theLayer);
+	delete theRegionList[rg];
+	theRegionList.remove(rg);
+
+	return true;
 }
 
 bool ImportExportOsmBin::clearTile(qint32 tile, MapDocument* d, OsbMapLayer* theLayer)
 {
-	Q_UNUSED(d);
+	int y = int(tile / NUM_TILES);
+	int x = (tile % NUM_TILES);
+	int rg = (y * NUM_REGIONS / NUM_TILES) * NUM_REGIONS + (x * NUM_REGIONS / NUM_TILES);
+	if (!theRegionList.contains(rg))
+		return true;
 
-	QDataStream ds(Device);
-	bool OK = true;
-
-	for (qint32 i=0; i<theTileIndex[tile].size(); ++i) {
-		MapFeature* F = theTileIndex[tile][i];
-		if (F->layer() == theLayer) {
-			int theCount = --featRefCount[F];
-			if (!(theCount)) {
-				theLayer->remove(F);
-				delete F;
-				featRefCount.remove(F);
-			}
-		} else {
-			featRefCount.remove(F);
-		}
-	}
-	theTileIndex.remove(tile);
-
-	//qDebug() << "--------- " << tile;
-	//QMapIterator<MapFeature*, quint32> itN(featRefCount);
-	//while(itN.hasNext()) {
-	//	itN.next();
-
-	//	if (itN.value() > 1)
-	//		qDebug() << itN.key()->id() << " - " << itN.value();
-	//}
-
-	return OK;
+	return theRegionList[rg]->clearTile(tile, d, theLayer);
 }
 
-MapFeature* ImportExportOsmBin::getFeature(MapDocument* d, OsbMapLayer* theLayer, quint64 ref)
+MapFeature* ImportExportOsmBin::getFeature(OsbRegion* osr, MapDocument* d, OsbMapLayer* theLayer, quint64 ref)
 {
-	QDataStream ds(Device);
+	QDataStream ds(osr->device);
 	MapFeature* F;
 	qint8 c;
 	quint64 id;
-	quint64 cur_pos = Device->pos();
+	quint64 cur_pos = osr->device->pos();
 
-	Device->seek(ref);
+	osr->device->seek(ref);
 	ds >> c;
 	ds >> id;
 	switch (c) {
@@ -559,10 +673,11 @@ MapFeature* ImportExportOsmBin::getFeature(MapDocument* d, OsbMapLayer* theLayer
 			F = d->getFeature(QString("rel_%1").arg(QString::number(id)));
 			break;
 		default:
+			F = NULL;
 			Q_ASSERT(false);
 	}
 	if (F && (F->lastUpdated() != MapFeature::NotYetDownloaded)) {
-		Device->seek(cur_pos);
+		osr->device->seek(cur_pos);
 		return F;
 	}
 
@@ -587,35 +702,33 @@ MapFeature* ImportExportOsmBin::getFeature(MapDocument* d, OsbMapLayer* theLayer
 	return F;
 }
 
-MapFeature* ImportExportOsmBin::getFeature(MapDocument* d, OsbMapLayer* theLayer)
+MapFeature* ImportExportOsmBin::getFeature(OsbRegion* osr, MapDocument* d, OsbMapLayer* theLayer)
 {
-	QDataStream ds(Device);
-	MapFeature* F;
+	QDataStream ds(osr->device);
+	MapFeature* F, * oF;
 	qint8 c;
 	quint64 id;
 
 	ds >> c;
 	ds >> id;
-	//switch (c) {
-	//	case 'N':
-	//		F = d->getFeature(QString("node_%1").arg(QString::number(id)));
-	//		break;
-	//	case 'R':
-	//		F = d->getFeature(QString("way_%1").arg(QString::number(id)));
-	//		break;
-	//	case 'L':
-	//		F = d->getFeature(QString("rel_%1").arg(QString::number(id)));
-	//		break;
-	//	default:
-	//		Q_ASSERT(false);
-	//}
-	//if (F && (F->lastUpdated() != MapFeature::NotYetDownloaded)) {
-	//	Device->seek(cur_pos);
-	//	return F;
-	//}
-#ifndef NDEBUG
-	quint32 pos = Device->pos(); //1097543
-#endif
+	switch (c) {
+		case 'N':
+			oF = d->getFeature(QString("node_%1").arg(QString::number(id)));
+			break;
+		case 'R':
+			oF = d->getFeature(QString("way_%1").arg(QString::number(id)));
+			break;
+		case 'L':
+			oF = d->getFeature(QString("rel_%1").arg(QString::number(id)));
+			break;
+		default:
+			oF = NULL;
+			Q_ASSERT(false);
+	}
+	if (oF && (!(oF->isNull()))) {
+		theLayer = NULL;
+	}
+
 	switch (c) {
 		case 'N':
 			F = TrackPoint::fromBinary(d, theLayer, ds, c, id);
@@ -630,15 +743,18 @@ MapFeature* ImportExportOsmBin::getFeature(MapDocument* d, OsbMapLayer* theLayer
 			tagsFromBinary(F, ds);
 			break;
 		default:
+			F = NULL;
 			Q_ASSERT(false);
 	}
 
-	//Device->seek(cur_pos);
-	return F;
+	if (oF && (!(oF->isNull())))
+		return oF;
+	else
+		return F;
 }
 
 // export
-bool ImportExportOsmBin::export_(const QVector<MapFeature *>& featList)
+bool ImportExportOsmBin::export_(const QList<MapFeature*>& featList)
 {
 	QDataStream ds(Device);
 	//theRegionToc.resize(TILE_WIDTH / REGION_WIDTH);
@@ -663,6 +779,31 @@ bool ImportExportOsmBin::export_(const QVector<MapFeature *>& featList)
 	return true;
 }
 
+bool ImportExportOsmBin::export_(const QList<MapFeature*>& featList, quint32 rg)
+{
+	QDataStream ds(Device);
+	//theRegionToc.resize(TILE_WIDTH / REGION_WIDTH);
+
+	if(! IImportExport::export_(featList) ) return false;
+
+	if (! prepare() ) return false;
+	if (! writeHeader(ds) ) return false;
+	if (! writeTagLists(ds) ) return false;
+
+	//if (! writeNodes(ds) ) return false;
+	//if (! writeRoads(ds) ) return false;
+	//if (! writeRelations(ds) ) return false;
+
+	if (! writeIndex(ds, rg) ) return false;
+
+	Device->seek(HEADER_SIZE);
+	ds << tocPos;
+	ds << tagKeysPos;
+	ds << tagValuesPos;
+
+	return true;
+}
+
 // import the  input
 bool ImportExportOsmBin::import(MapLayer* aLayer)
 {
@@ -681,3 +822,17 @@ bool ImportExportOsmBin::import(MapLayer* aLayer)
 
 	return true;
 }
+
+bool ImportExportOsmBin::writeWorld(QDataStream& ds)
+{
+	if (! writeHeader(ds) ) return false;
+	if (! writeIndex(ds) ) return false;
+
+	ds.device()->seek(HEADER_SIZE);
+	ds << tocPos;
+	ds << (qint64)0;
+	ds << (qint64) 0;
+
+	return true;
+}
+
