@@ -27,7 +27,7 @@ public:
 	// import the  input
 	virtual bool import(MapLayer* aLayer);
 	// export
-	virtual bool export_(const QVector<MapFeature *>& featList);
+	virtual bool export_(const QList<MapFeature *>& featList);
 
 private:
 	TrackMapLayer* theLayer;
@@ -35,6 +35,7 @@ private:
 	bool importGSA (QString line);
 	bool importGSV (QString line);
 	bool importGGA (QString line);
+	bool importGLL (QString line);
 	TrackPoint* importRMC (QString line);
 
 	double curAltitude;
