@@ -31,6 +31,11 @@ public:
 
 public:
 	void setHtml(QString html);
+	QString getHtml();
+	void setHoverHtml(QString html);
+	void unsetHoverHtml();
+        void changeEvent(QEvent *);
+        void retranslateUi();
 
 private slots:
 	void on_anchorClicked(const QUrl & link);
@@ -38,6 +43,7 @@ private slots:
 private:
 	MainWindow* Main;
 	QTextBrowser* theText;
+	QString currentHtml;
 };
 
 #endif
