@@ -11,18 +11,21 @@
 #include "Map/TrackSegment.h"
 #include "Utils/SlippyMapWidget.h"
 #include "Preferences/MerkaartorPreferences.h"
+#include "ImportExport/ImportExportOsmBin.h"
+
+#include "IProgressWindow.h"
 
 #include <ui_DownloadMapDialog.h>
 
-#include <QtCore/QBuffer>
-#include <QtCore/QSettings>
-#include <QtCore/QTimer>
-#include <QtGui/QComboBox>
-#include <QtGui/QMainWindow>
-#include <QtGui/QMessageBox>
-#include <QtGui/QProgressBar>
-#include <QtGui/QProgressDialog>
-#include <QtGui/QStatusBar>
+#include <QBuffer>
+#include <QSettings>
+#include <QTimer>
+#include <QComboBox>
+#include <QMessageBox>
+#include <QProgressBar>
+#include <QLabel>
+#include <QProgressDialog>
+#include <QStatusBar>
 #include <QInputDialog>
 
 #include "zlib/zlib.h"
@@ -36,7 +39,7 @@
 Downloader::Downloader(const QString& aWeb, const QString& aUser, const QString& aPwd, bool aUseProxy, const QString& aProxyHost, int aProxyPort)
 : Port(80), Web(aWeb), User(aUser), Password(aPwd),
   UseProxy(aUseProxy), ProxyHost(aProxyHost), ProxyPort(aProxyPort), Id(0),
-  Error(false), Animator(0), AnimatorBar(0), AnimationTimer(0)
+  Error(false), AnimatorLabel(0), AnimatorBar(0), AnimationTimer(0)
 {
 	int p = Web.lastIndexOf(':');
 	if (p != -1)
@@ -44,11 +47,13 @@ Downloader::Downloader(const QString& aWeb, const QString& aUser, const QString&
 		Port = Web.right(Web.length()-(p+1)).toUInt();
 		Web = Web.left(p);
 	}
+
 	Request.setHost(Web,Port);
-	Request.setUser(User,Password);
+	//IdAuth = Request.setUser(User.toUtf8(), Password.toUtf8());
 //	connect(&Request,SIGNAL(done(bool)), this,SLOT(allDone(bool)));
 	connect(&Request,SIGNAL(requestFinished(int, bool)),this,SLOT(on_requestFinished(int, bool)));
 	connect(&Request,SIGNAL(dataReadProgress(int, int)), this,SLOT(progress(int, int)));
+	connect(&Request, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)), this, SLOT(on_responseHeaderReceived(const QHttpResponseHeader &)));
 }
 
 
@@ -58,21 +63,22 @@ void Downloader::animate()
 		AnimatorBar->setValue((AnimatorBar->value()+1) % AnimatorBar->maximum());
 }
 
-void Downloader::setAnimator(QProgressDialog *anAnimator,  QProgressBar* aBar, bool anAnimate)
+void Downloader::setAnimator(QProgressDialog *anAnimator, QLabel* anAnimatorLabel, QProgressBar* anAnimatorBar, bool anAnimate)
 {
-	Animator = anAnimator;
-	AnimatorBar = aBar;
-	if (Animator && anAnimate)
+	AnimatorLabel = anAnimatorLabel;
+	AnimatorBar = anAnimatorBar;
+	if (AnimatorBar && anAnimate)
 	{
 		if (AnimationTimer)
 			delete AnimationTimer;
 		AnimationTimer = new QTimer(this);
 		connect(AnimationTimer,SIGNAL(timeout()),this,SLOT(animate()));
 	}
-	if (Animator)
+	if (AnimatorBar)
 	{
-		Animator->setValue(0);
-		connect(Animator,SIGNAL(canceled()),this,SLOT(on_Cancel_clicked()));
+		AnimatorBar->setValue(0);
+		if (anAnimator)
+			connect(anAnimator,SIGNAL(canceled()),this,SLOT(on_Cancel_clicked()));
 		qApp->processEvents();
 	}
 }
@@ -173,6 +179,9 @@ QByteArray gzipDecode(const QByteArray& In)
 bool Downloader::go(const QString& url)
 {
 	if (Error) return false;
+
+	qDebug() << url;
+
 	if (AnimationTimer)
 		AnimationTimer->start(200);
 	Content.clear();
@@ -199,7 +208,7 @@ bool Downloader::go(const QString& url)
 	{
 		QMessageBox::information(0,tr("error"),Request.errorString());
 	}
-	if (Content.size() != (int)Request.lastResponse().contentLength())
+	if (Request.lastResponse().hasContentLength() && Content.size() != (int)Request.lastResponse().contentLength())
 	{
 		QMessageBox::information(0,tr("didn't download enough"),QString("%1 %2").arg(Content.size()).arg(Request.lastResponse().contentLength()));
 	}
@@ -218,10 +227,10 @@ bool Downloader::go(const QString& url)
 	showDebug("GET", url, QByteArray() ,Content);
 #endif
 	Result = Request.lastResponse().statusCode();
+	LocationText = Request.lastResponse().value("Location");
 	ResultText = Request.lastResponse().reasonPhrase();
- 	ErrorText = Request.lastResponse().value("Error");
-	delete AnimationTimer;
-	AnimationTimer = 0;
+	ErrorText = Request.lastResponse().value("Error");
+	SAFE_DELETE(AnimationTimer);
 	return !Error;
 }
 
@@ -238,6 +247,11 @@ bool Downloader::request(const QString& Method, const QString& URL, const QStrin
 		Header.setValue("Host",Web);
 	else
 		Header.setValue("Host",Web+':'+QString::number(Port));
+	
+	QString auth = QString("%1:%2").arg(User).arg(Password);
+	QByteArray ba_auth = auth.toUtf8().toBase64();
+	Header.setValue("Authorization", QString("Basic %1").arg(QString(ba_auth)));
+
 	Content.clear();
 	Id = Request.request(Header,ba);
 
@@ -270,6 +284,24 @@ QByteArray& Downloader::content()
 	return Content;
 }
 
+void Downloader::on_responseHeaderReceived(const QHttpResponseHeader & hdr)
+{
+	switch (hdr.statusCode()) {
+		//case 200:
+		//	break;
+		//case 406:
+		//	QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","Duplicate key"));
+		//	break;
+		//case 413:
+		//	QMessageBox::critical(NULL,QApplication::translate("MerkaartorPreferences","Preferences upload failed"), QApplication::translate("MerkaartorPreferences","More than 150 preferences"));
+		//	break;
+		default:
+			qDebug() << hdr.statusCode();
+			qDebug() << hdr.reasonPhrase();
+			break;
+	}
+}
+
 void Downloader::on_requestFinished(int anId, bool anError)
 {
 	if (anError)
@@ -280,16 +312,15 @@ void Downloader::on_requestFinished(int anId, bool anError)
 
 void Downloader::progress(int done, int total)
 {
-	if (Animator)
+	if (AnimatorLabel && AnimatorBar)
 	{
 		if (done < 10240)
-			Animator->setLabelText(tr("Downloading from OSM (%1 bytes)").arg(done));
+			AnimatorLabel->setText(tr("Downloading from OSM (%n bytes)", "", done));
 		else
-			Animator->setLabelText(tr("Downloading from OSM (%1 kBytes)").arg(done/1024));
+			AnimatorLabel->setText(tr("Downloading from OSM (%n kBytes)", "", (done/1024)));
 		if (AnimationTimer && total != 0)
 		{
-			delete AnimationTimer;
-			AnimationTimer = 0;
+			SAFE_DELETE(AnimationTimer);
 			AnimatorBar->setMaximum(total);
 		}
 		if (!AnimationTimer && AnimatorBar)
@@ -312,14 +343,19 @@ const QString &Downloader::errorText()
 	return ErrorText;
 }
 
+const QString &Downloader::locationText()
+{
+	return LocationText;
+}
+
 QString Downloader::getURLToOpenChangeSet()
 {
-	return QString("/api/0.6/changeset/create");
+	return QString("/api/%1/changeset/create").arg(M_PREFS->apiVersion());
 }
 
 QString Downloader::getURLToCloseChangeSet(const QString& Id)
 {
-	return QString("/api/0.6/changeset/%1/close").arg(Id);
+	return QString("/api/%1/changeset/%2/close").arg(M_PREFS->apiVersion()).arg(Id);
 }
 
 QString Downloader::getURLToFetch(const QString &What)
@@ -338,77 +374,75 @@ QString Downloader::getURLToFetchFull(MapFeature* aFeature)
 	if (aFeature->getClass() == "Relation")
 		What = "relation";
 	QString Id = aFeature->xmlId();
-	QString URL = QString("/api/0.5/%1/%2/full");
-	return URL.arg(What).arg(Id);
+	QString URL = QString("/api/%1/%2/%3/full");
+	return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
 }
 
 QString Downloader::getURLToFetch(const QString &What, const QString& Id)
 {
-	QString URL = QString("/api/0.5/%1/%2");
-	return URL.arg(What).arg(Id);
+	QString URL = QString("/api/%1/%2/%3");
+	return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
 }
 
 QString Downloader::getURLToCreate(const QString &What)
 {
-	if (MerkaartorPreferences::instance()->use06Api())
-		return QString("/api/0.6/%1/create").arg(What);
-	QString URL = QString("/api/0.5/%1/create");
-	return URL.arg(What);
+	QString URL = QString("/api/%1/%2/create");
+	return URL.arg(M_PREFS->apiVersion()).arg(What);
 }
 
 QString Downloader::getURLToUpdate(const QString &What, const QString& Id)
 {
-	if (MerkaartorPreferences::instance()->use06Api())
-		return QString("/api/0.6/%1/%2").arg(What).arg(Id);
-	QString URL = QString("/api/0.5/%1/%2");
-	return URL.arg(What).arg(Id);
+	QString URL = QString("/api/%1/%2/%3");
+	return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
 }
 
 QString Downloader::getURLToDelete(const QString &What, const QString& Id)
 {
-	if (MerkaartorPreferences::instance()->use06Api())
-		return QString("/api/0.6/%1/%2").arg(What).arg(Id);
-	QString URL = QString("/api/0.5/%1/%2");
-	return URL.arg(What).arg(Id);
+	QString URL = QString("/api/%1/%2/%3");
+	return URL.arg(M_PREFS->apiVersion()).arg(What).arg(Id);
 }
 
 QString Downloader::getURLToMap()
 {
-	if (MerkaartorPreferences::instance()->use06Api())
-		return QString("/api/0.6/map?bbox=%1,%2,%3,%4");
-	QString URL = QString("/api/0.5/map?bbox=%1,%2,%3,%4");
+	QString apiURL = QString("/api/%1").arg(M_PREFS->apiVersion());
+	QString URL = apiURL + QString("/map?bbox=%1,%2,%3,%4");
 	return URL;
 }
 
 QString Downloader::getURLToTrackPoints()
 {
-	QString URL = QString("/api/0.5/trackpoints?bbox=%1,%2,%3,%4&page=%5");
+	QString URL = QString("/api/%1").arg(M_PREFS->apiVersion()) + QString("/trackpoints?bbox=%1,%2,%3,%4&page=%5");
 	return URL;
 }
 
-bool downloadOSM(QMainWindow* aParent, const QUrl& theUrl, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort, MapDocument* theDocument, MapLayer* theLayer)
+bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort, MapDocument* theDocument, MapLayer* theLayer)
 {
 	QString aWeb = theUrl.host();
 	QString URL = theUrl.path();
 	Downloader Rcv(aWeb, aUser, aPassword, UseProxy, ProxyHost, ProxyPort);
 
-	QProgressDialog* ProgressDialog = new QProgressDialog(aParent);
-	ProgressDialog->setWindowModality(Qt::ApplicationModal);
-	QProgressBar* Bar = new QProgressBar(ProgressDialog);
-	Bar->setTextVisible(false);
-	ProgressDialog->setBar(Bar);
-	ProgressDialog->setMinimumDuration(0);
-	ProgressDialog->setLabelText(QApplication::translate("Downloader","Downloading from OSM (connecting)"));
-	ProgressDialog->setMaximum(11);
-	Rcv.setAnimator(ProgressDialog,Bar,true);
+	IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(aParent);
+	if (!aProgressWindow)
+		return false;
 
+	QProgressDialog* dlg = aProgressWindow->getProgressDialog();
+
+	QProgressBar* Bar = aProgressWindow->getProgressBar();
+	Bar->setTextVisible(false);
+	Bar->setMaximum(11);
+
+	QLabel* Lbl = aProgressWindow->getProgressLabel();
+	Lbl->setText(QApplication::translate("Downloader","Downloading from OSM (connecting)"));
+
+	if (dlg)
+		dlg->show();
+
+	Rcv.setAnimator(dlg, Lbl, Bar, true);
 	if (!Rcv.go(URL))
 	{
 		aParent->setCursor(QCursor(Qt::ArrowCursor));
-		delete ProgressDialog;
 		return false;
 	}
-	delete ProgressDialog;
 #ifdef DEBUG_MAPCALL_ONLY
 	showDebug("GET", URL,QByteArray(), Rcv.content());
 #endif
@@ -417,6 +451,22 @@ bool downloadOSM(QMainWindow* aParent, const QUrl& theUrl, const QString& aUser,
 	{
 	case 200:
 		break;
+	case 301:
+	case 302:
+	case 307:
+		aWeb = Rcv.locationText();
+		if (!aWeb.isEmpty()) {
+			QUrl aURL = theUrl;
+			aURL.setHost(aWeb);
+			return downloadOSM(aParent, aURL, aUser, aPassword, UseProxy, ProxyHost, ProxyPort, theDocument, theLayer);
+		} else {
+			QString msg = QApplication::translate("Downloader","Unexpected http status code (%1)\nServer message is '%2'").arg(x).arg(Rcv.resultText());
+			if (!Rcv.errorText().isEmpty())
+				msg += QApplication::translate("Downloader", "\nAPI message is '%1'").arg(Rcv.errorText());
+			QMessageBox::warning(aParent,QApplication::translate("Downloader","Download failed"), msg);
+			return false;
+		}
+		break;
 	case 401:
 		QMessageBox::warning(aParent,QApplication::translate("Downloader","Download failed"),QApplication::translate("Downloader","Username/password invalid"));
 		return false;
@@ -432,7 +482,7 @@ bool downloadOSM(QMainWindow* aParent, const QUrl& theUrl, const QString& aUser,
 	return OK;
 }
 
-bool downloadOSM(QMainWindow* aParent, const QString& aWeb, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort, const CoordBox& aBox , MapDocument* theDocument, MapLayer* theLayer)
+bool downloadOSM(QWidget* aParent, const QString& aWeb, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort, const CoordBox& aBox , MapDocument* theDocument, MapLayer* theLayer)
 {
 	if (checkForConflicts(theDocument))
 	{
@@ -451,26 +501,63 @@ bool downloadOSM(QMainWindow* aParent, const QString& aWeb, const QString& aUser
 	return downloadOSM(aParent, theUrl, aUser, aPassword, UseProxy, ProxyHost, ProxyPort, theDocument, theLayer);
 }
 
+bool downloadOSM(QWidget* Main, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort , const quint32 region , MapDocument* theDocument, MapLayer* theLayer)
+{
+	Q_UNUSED(aUser)
+	Q_UNUSED(aPassword)
+	int y = int(region / NUM_REGIONS); //2565024
+	int x = (region % NUM_REGIONS);
+	CoordBox Clip = CoordBox(
+		Coord (y * REGION_WIDTH - INT_MAX, x * REGION_WIDTH - INT_MAX  ),
+		Coord ((y+1) * REGION_WIDTH - INT_MAX, (x+1) * REGION_WIDTH - INT_MAX )
+		);
 
-bool downloadTracksFromOSM(QMainWindow* Main, const QString& aWeb, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort , const CoordBox& aBox , MapDocument* theDocument)
+	QString osmWebsite, osmUser, osmPwd;
+	osmWebsite = M_PREFS->getOsmWebsite();
+	osmUser = M_PREFS->getOsmUser();
+	osmPwd = M_PREFS->getOsmPassword();
+
+	return downloadOSM(Main,osmWebsite,osmUser,osmPwd,UseProxy,ProxyHost,ProxyPort,Clip,theDocument,theLayer);
+
+// This code will never be reached. Can It be removed?
+//	QString aUrl(QString("http://%1/api/0.5/*[bbox=%2,%3,%4,%5]")
+//		.arg(M_PREFS->getXapiWebSite())
+//		.arg(intToAng(Clip.bottomLeft().lon())).arg(intToAng(Clip.bottomLeft().lat()))
+//		.arg(intToAng(Clip.topRight().lon())).arg(intToAng(Clip.topRight().lat()))
+//		);
+//	qDebug() << aUrl;
+//	QUrl theUrl(aUrl);
+//
+//	return downloadOSM(Main,theUrl,aUser,aPassword,UseProxy,ProxyHost,ProxyPort,theDocument, theLayer);
+}
+
+bool downloadTracksFromOSM(QWidget* Main, const QString& aWeb, const QString& aUser, const QString& aPassword, bool UseProxy, const QString& ProxyHost, int ProxyPort , const CoordBox& aBox , MapDocument* theDocument)
 {
 	Downloader theDownloader(aWeb, aUser, aPassword, UseProxy, ProxyHost, ProxyPort);
 	QVector<TrackMapLayer*> theTracklayers;
 	//TrackMapLayer* trackLayer = new TrackMapLayer(QApplication::translate("Downloader","Downloaded tracks"));
 	//theDocument->add(trackLayer);
-	QProgressDialog ProgressDialog(Main);
-	ProgressDialog.setWindowModality(Qt::ApplicationModal);
-	QProgressBar* Bar = new QProgressBar(&ProgressDialog);
+
+	IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(Main);
+	if (!aProgressWindow)
+		return false;
+
+	QProgressDialog* dlg = aProgressWindow->getProgressDialog();
+
+	QProgressBar* Bar = aProgressWindow->getProgressBar();
 	Bar->setTextVisible(false);
-	ProgressDialog.setBar(Bar);
-	ProgressDialog.setMinimumDuration(0);
-	ProgressDialog.setMaximum(11);
-	ProgressDialog.setValue(1);
-	ProgressDialog.show();
-	theDownloader.setAnimator(&ProgressDialog,Bar,true);
+	Bar->setMaximum(11);
+
+	QLabel* Lbl = aProgressWindow->getProgressLabel();
+	Lbl->setText(QApplication::translate("Downloader","Parsing XML"));
+
+	if (dlg)
+		dlg->show();
+
+	theDownloader.setAnimator(dlg,Lbl,Bar,true);
 	for (unsigned int Page=0; ;++Page)
 	{
-		ProgressDialog.setLabelText(QApplication::translate("Downloader","Downloading trackpoints %1-%2").arg(Page*5000+1).arg(Page*5000+5000));
+		Lbl->setText(QApplication::translate("Downloader","Downloading trackpoints %1-%2").arg(Page*5000+1).arg(Page*5000+5000));
 		QString URL = theDownloader.getURLToTrackPoints();
 		URL = URL.arg(intToAng(aBox.bottomLeft().lon())).
 				arg(intToAng(aBox.bottomLeft().lat())).
@@ -497,16 +584,15 @@ bool downloadTracksFromOSM(QMainWindow* Main, const QString& aWeb, const QString
 bool checkForConflicts(MapDocument* theDocument)
 {
 	for (FeatureIterator it(theDocument); !it.isEnd(); ++it) {
-		if (it.get()->layer() == theDocument->getTrashLayer())
-			continue;
 		if (it.get()->lastUpdated() == MapFeature::OSMServerConflict)
 			return true;
 	}
 	return false;
 }
 
-bool downloadMoreOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* theDocument)
+bool downloadMoreOSM(QWidget* aParent, const CoordBox& aBox , MapDocument* theDocument)
 {
+	MainWindow* Main = dynamic_cast<MainWindow*>(aParent);
 	MapLayer* theLayer;
 	if (!theDocument->getLastDownloadLayer()) {
 		theLayer = new DrawingMapLayer(QApplication::translate("Downloader","%1 download").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
@@ -526,28 +612,36 @@ bool downloadMoreOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* th
 	useProxy = MerkaartorPreferences::instance()->getProxyUse();
 	proxyHost = MerkaartorPreferences::instance()->getProxyHost();
 	proxyPort = MerkaartorPreferences::instance()->getProxyPort();
-	aParent->view()->setUpdatesEnabled(false);
+	Main->view()->setUpdatesEnabled(false);
 
 	bool OK = true;
 	OK = downloadOSM(aParent,osmWebsite,osmUser,osmPwd,useProxy,proxyHost,proxyPort,aBox,theDocument,theLayer);
-	aParent->view()->setUpdatesEnabled(true);
+	Main->view()->setUpdatesEnabled(true);
 	if (OK)
 	{
 		theDocument->setLastDownloadLayer(theLayer);
-		theDocument->addDownloadBox(aBox);
-		aParent->view()->projection().setViewport(aBox,aParent->view()->rect());
-		aParent->invalidateView();
+		theDocument->addDownloadBox(theLayer, aBox);
+		// Don't jump around on Download More
+		// aParent->view()->projection().setViewport(aBox,aParent->view()->rect());
+		Main->invalidateView();
+	} else
+	{
+		if (theLayer != theDocument->getLastDownloadLayer()) {
+			theDocument->remove(theLayer);
+			delete theLayer;
+		}
 	}
 	return OK;
 }
 
-bool downloadOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* theDocument)
+bool downloadOSM(QWidget* aParent, const CoordBox& aBox , MapDocument* theDocument)
 {
 	QString osmWebsite, osmUser, osmPwd, proxyHost;
 	int proxyPort;
 	bool useProxy;
 	static bool DownloadRaw = false;
 
+	MainWindow* Main = dynamic_cast<MainWindow*>(aParent);
 	QDialog * dlg = new QDialog(aParent);
 
 	osmWebsite = MerkaartorPreferences::instance()->getOsmWebsite();
@@ -564,14 +658,18 @@ bool downloadOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* theDoc
 #ifndef _MOBILE
 	SlippyMap->setMinimumHeight(256);
 #endif
+	CoordBox Clip(aBox);
+	SlippyMap->setViewportArea(Clip.toQRectF());
 	ui.vboxLayout1->addWidget(SlippyMap);
 	QObject::connect(SlippyMap, SIGNAL(redraw()), ui.FromMap, SLOT(toggle()));
-	QStringList Bookmarks = MerkaartorPreferences::instance()->getBookmarks();
-	for (int i=0; i<Bookmarks.size(); i+=5)
-		ui.Bookmarks->addItem(Bookmarks[i]);
+	QMapIterator<QString, CoordBox> i(M_PREFS->getBookmarks());
+	while (i.hasNext()) {
+		i.next();
+		ui.Bookmarks->addItem(i.key());
+	}
 	ui.IncludeTracks->setChecked(DownloadRaw);
 	ui.ResolveRelations->setChecked(M_PREFS->getResolveRelations());
-	bool OK = true, retry = true, directAPI = false;
+	bool OK = true, retry = true, directAPI = false, Regional=false;
 	while (retry) {
 		retry = false;
 #ifdef _MOBILE
@@ -580,12 +678,9 @@ bool downloadOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* theDoc
 		if (dlg->exec() == QDialog::Accepted)
 		{
 			DownloadRaw = false;
-			CoordBox Clip(Coord(0,0),Coord(0,0));
 			if (ui.FromBookmark->isChecked())
 			{
-				unsigned int idx = ui.Bookmarks->currentIndex()*5+1;
-				Clip = CoordBox(Coord(angToInt(Bookmarks[idx].toDouble()),angToInt(Bookmarks[idx+1].toDouble())),
-					Coord(angToInt(Bookmarks[idx+2].toDouble()),angToInt(Bookmarks[idx+3].toDouble())));
+				Clip = M_PREFS->getBookmarks()[ui.Bookmarks->currentText()];
 			}
 			else if (ui.FromView->isChecked())
 			{
@@ -597,61 +692,71 @@ bool downloadOSM(MainWindow* aParent, const CoordBox& aBox , MapDocument* theDoc
 				if (link.contains("/api/")) {
 					directAPI=true;
 				} else {
-					QUrl url = QUrl(link); 
-					double lat = url.queryItemValue("lat").toDouble(); 
-					double lon = url.queryItemValue("lon").toDouble(); 
-					int zoom = url.queryItemValue("zoom").toInt();
+					link.toUInt(&Regional);
+					if (!Regional) {
+						QUrl url = QUrl(link); 
+						double lat = url.queryItemValue("lat").toDouble(); 
+						double lon = url.queryItemValue("lon").toDouble(); 
+						int zoom = url.queryItemValue("zoom").toInt();
 
-					if (zoom <= 10) {
-						QMessageBox::warning(dlg, QApplication::translate("Downloader", "Zoom factor too low"),
-							QApplication::translate("Downloader", "Please use a higher zoom factor!"));
-						retry = true;
-					}
-					else {
-						if (zoom < 1 || zoom > 18) // use default when not in bounds
-							zoom = 15;
+						if (zoom <= 10) {
+							QMessageBox::warning(dlg, QApplication::translate("Downloader", "Zoom factor too low"),
+								QApplication::translate("Downloader", "Please use a higher zoom factor!"));
+							retry = true;
+						}
+						else {
+							if (zoom < 1 || zoom > 18) // use default when not in bounds
+								zoom = 15;
 
-						/* term to calculate the angle from the zoom-value */
-						double zoomLat = 360.0 / (double)(1 << zoom);
-						double zoomLon = zoomLat / fabs(cos(angToRad(lat)));
-						/* the following line is equal to the line above. (just for explanation) */
-						//double zoomLon = zoomLat / aParent->view()->projection().latAnglePerM() * aParent->view()->projection().lonAnglePerM(angToRad(lat));
+							/* term to calculate the angle from the zoom-value */
+							double zoomLat = 360.0 / (double)(1 << zoom);
+							double zoomLon = zoomLat / fabs(cos(angToRad(lat)));
+							/* the following line is equal to the line above. (just for explanation) */
+							//double zoomLon = zoomLat / aParent->view()->projection().latAnglePerM() * aParent->view()->projection().lonAnglePerM(angToRad(lat));
 
-						/* the OSM link contains the coordinates from the middle of the visible map so we have to add and sub zoomLon/zoomLat */
-						Clip = CoordBox(Coord(angToInt(lat-zoomLat), angToInt(lon-zoomLon)), Coord(angToInt(lat+zoomLat), angToInt(lon+zoomLon)));
+							/* the OSM link contains the coordinates from the middle of the visible map so we have to add and sub zoomLon/zoomLat */
+							Clip = CoordBox(Coord(angToInt(lat-zoomLat), angToInt(lon-zoomLon)), Coord(angToInt(lat+zoomLat), angToInt(lon+zoomLon)));
+						}
 					}
 				}
 			}
 			else if (ui.FromMap->isChecked())
 			{
 				QRect R(SlippyMap->viewArea());
-				Clip = CoordBox(Coord(R.x(), R.y()), Coord(R.x()+R.width(), R.y()+R.height()));
+				Clip = CoordBox(Coord(R.y(), R.x()), Coord(R.y()+R.height(), R.x()+R.width()));
 			}
 			if (retry) continue;
-			aParent->view()->setUpdatesEnabled(false);
+			Main->view()->setUpdatesEnabled(false);
 			MapLayer* theLayer = new DrawingMapLayer(QApplication::translate("Downloader","%1 download").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
 			theDocument->add(theLayer);
 			M_PREFS->setResolveRelations(ui.ResolveRelations->isChecked());
 			if (directAPI)
 				OK = downloadOSM(aParent,QUrl(ui.Link->text()),osmUser,osmPwd,useProxy,proxyHost,proxyPort,theDocument,theLayer);
+			else
+			if (Regional)
+				OK = downloadOSM(aParent,osmUser,osmPwd,useProxy,proxyHost,proxyPort,ui.Link->text().toUInt(),theDocument,theLayer);
 			else
 				OK = downloadOSM(aParent,osmWebsite,osmUser,osmPwd,useProxy,proxyHost,proxyPort,Clip,theDocument,theLayer);
 			if (OK && ui.IncludeTracks->isChecked())
 				OK = downloadTracksFromOSM(aParent,osmWebsite,osmUser,osmPwd,useProxy,proxyHost,proxyPort, Clip,theDocument);
-			aParent->view()->setUpdatesEnabled(true);
+			Main->view()->setUpdatesEnabled(true);
 			if (OK)
 			{
 				theDocument->setLastDownloadLayer(theLayer);
-				theDocument->addDownloadBox(Clip);
-				aParent->view()->projection().setViewport(Clip,aParent->view()->rect());
-				aParent->invalidateView();
+				theDocument->addDownloadBox(theLayer, Clip);
+                if (directAPI)
+                    Main->on_viewZoomAllAction_triggered();
+                else
+                    Main->view()->projection().setViewport(Clip,Main->view()->rect());
+				Main->invalidateView();
 			} else {
 				retry = true;
 				theDocument->remove(theLayer);
-				delete theLayer;
+				SAFE_DELETE(theLayer);
 			}
 		}
 	}
 	delete dlg;
 	return OK;
 }
+
