@@ -18,6 +18,9 @@
 #include "QMapControl/mapadapter.h"
 #include "QMapControl/osmmapadapter.h"
 #include "QMapControl/wmsmapadapter.h"
+#ifdef USE_GDAL
+	#include "ImportExport/ImportExportSHP.h"
+#endif
 #ifdef YAHOO
 	#include "QMapControl/yahoolegalmapadapter.h"
 #endif
@@ -43,8 +46,6 @@
 #include <map>
 #include <vector>
 
-#define SAFE_DELETE(x) {delete (x); x = NULL;}
-
 /* MAPLAYER */
 
 class MapLayerPrivate
@@ -58,19 +59,22 @@ public:
 		theWidget = NULL;
 		selected = false;
 		Enabled = true;
+		Readonly = false;
 	}
 	~MapLayerPrivate()
 	{
-		for (unsigned int i=0; i<Features.size(); ++i)
-			delete Features[i];
+		for (int i=0; i<Features.size(); ++i)
+			if (Features[i])
+				delete Features[i];
 	}
-	std::vector<MapFeature*> Features;
-	QHash<QString, MapFeature*> IdMap;
+	QList<MapFeaturePtr> Features;
+	QHash<QString, MapFeaturePtr> IdMap;
 	QString Name;
 	QString Description;
 	bool Visible;
 	bool selected;
 	bool Enabled;
+	bool Readonly;
 	LayerWidget* theWidget;
 	qreal alpha;
 	unsigned int dirtyLevel;
@@ -123,7 +127,7 @@ MapLayer::MapLayer(const QString& aName)
 }
 
 MapLayer::MapLayer(const MapLayer&)
-: p(0)
+: QObject(), p(0)
 {
 }
 
@@ -134,7 +138,9 @@ MapLayer::~MapLayer()
 
 void MapLayer::sortRenderingPriority(double aPixelPerM)
 {
-	if (!p->RenderPriorityUpToDate || (aPixelPerM != p->RenderPriorityForPixelPerM) )
+	// No need to resort after a zoom, is there?
+	// if (!p->RenderPriorityUpToDate || (aPixelPerM != p->RenderPriorityForPixelPerM) )
+	if (!p->RenderPriorityUpToDate)
 		p->sortRenderingPriority(aPixelPerM);
 }
 
@@ -190,6 +196,15 @@ bool MapLayer::isEnabled() const
 	return p->Enabled;
 }
 
+void MapLayer::setReadonly(bool b) {
+	p->Readonly = b;
+}
+
+bool MapLayer::isReadonly() const
+{
+	return p->Readonly;
+}
+
 void MapLayer::add(MapFeature* aFeature)
 {
 	p->Features.push_back(aFeature);
@@ -211,19 +226,32 @@ void MapLayer::notifyIdUpdate(const QString& id, MapFeature* aFeature)
 
 void MapLayer::remove(MapFeature* aFeature)
 {
-	std::vector<MapFeature*>::iterator i = std::find(p->Features.begin(),p->Features.end(), aFeature);
-	if (i != p->Features.end())
+	int i = p->Features.indexOf(aFeature);
+	if (i != -1)
 	{
-		p->Features.erase(i);
+		p->Features.removeAt(i);
 		aFeature->setLayer(0);
 		notifyIdUpdate(aFeature->id(),0);
 		p->RenderPriorityUpToDate = false;
 	}
 }
 
+void MapLayer::deleteFeature(MapFeature* aFeature)
+{
+	int i = p->Features.indexOf(aFeature);
+	if (i != -1)
+	{
+		p->Features.removeAt(i);
+		aFeature->setLayer(0);
+		notifyIdUpdate(aFeature->id(),0);
+		p->RenderPriorityUpToDate = false;
+	}
+	delete aFeature;
+}
+
 void MapLayer::clear()
 {
-	std::vector<MapFeature*>::iterator i;
+	QList<MapFeaturePtr>::iterator i;
 	for (i=p->Features.begin(); i != p->Features.end();)
 	{
 		(*i)->setLayer(0);
@@ -235,8 +263,8 @@ void MapLayer::clear()
 
 bool MapLayer::exists(MapFeature* F) const
 {
-	std::vector<MapFeature*>::iterator i = std::find(p->Features.begin(),p->Features.end(), F);
-	return i != p->Features.end();
+	int i = p->Features.indexOf(F);
+	return (i != -1);
 }
 
 unsigned int MapLayer::size() const
@@ -256,13 +284,23 @@ MapDocument* MapLayer::getDocument()
 
 int MapLayer::get(MapFeature* aFeature)
 {
-	for (unsigned int i=0; i<p->Features.size(); ++i)
+	for (int i=0; i<p->Features.size(); ++i)
 		if (p->Features[i] == aFeature)
 			return i;
 
 	return -1;
 }
 
+QList<MapFeature *> MapLayer::get()
+{
+	QList<MapFeature *> theList;
+	for (int i=0; i<p->Features.size(); ++i)
+		if (p->Features[i])
+			theList.append(p->Features[i]);
+	return theList;
+}
+
+
 MapFeature* MapLayer::get(unsigned int i)
 {
 	return p->Features[i];
@@ -270,7 +308,7 @@ MapFeature* MapLayer::get(unsigned int i)
 
 MapFeature* MapLayer::get(const QString& id, bool exact)
 {
-	QHash<QString, MapFeature*>::const_iterator i;
+	QHash<QString, MapFeaturePtr>::const_iterator i;
 	
 	i = p->IdMap.find(id);
 	if (i != p->IdMap.end())
@@ -292,7 +330,7 @@ MapFeature* MapLayer::get(const QString& id, bool exact)
 
 const MapFeature* MapLayer::get(unsigned int i) const
 {
-	if(i>=p->Features.size()) return 0;
+	if((int)i>=p->Features.size()) return 0;
 	return p->Features[i];
 }
 
@@ -333,9 +371,18 @@ const QString& MapLayer::id() const
 CoordBox MapLayer::boundingBox(const MapLayer* theLayer)
 {
 	if(theLayer->size()==0) return CoordBox(Coord(0,0),Coord(0,0));
-	CoordBox Box(theLayer->get(0)->boundingBox());
-	for (unsigned int i=1; i<theLayer->size(); ++i)
-		Box.merge(theLayer->get(i)->boundingBox());
+	CoordBox Box;
+	bool haveFirst = false;
+	for (unsigned int i=0; i<theLayer->size(); ++i) {
+		if (theLayer->get(i)->isDeleted())
+			continue;
+		if (haveFirst)
+			Box.merge(theLayer->get(i)->boundingBox());
+		else {
+			Box = theLayer->get(i)->boundingBox();
+			haveFirst = true;
+		}
+	}
 	return Box;
 }
 
@@ -359,6 +406,18 @@ unsigned int MapLayer::getDirtyLevel()
 	return p->dirtyLevel;
 }
 
+unsigned int MapLayer::getDirtySize()
+{
+	int dirtyObjects = 0;
+
+	QList<MapFeaturePtr>::const_iterator i;
+	for (i = p->Features.constBegin(); i != p->Features.constEnd(); i++)
+		if (!((*i)->isDeleted()) || ((*i)->isDeleted() && (*i)->hasOSMId()))
+			++dirtyObjects;
+
+	return dirtyObjects;
+}
+
 bool MapLayer::canDelete()
 {
 	return (p->dirtyLevel == 0);
@@ -443,10 +502,10 @@ bool DrawingMapLayer::toXML(QDomElement xParent, QProgressDialog & progress)
 		S += QString().number(intToAng(layBB.topRight().lat()),'f',6) + ",";
 		S += QString().number(intToAng(layBB.topRight().lon()),'f',6);
 		bb.setAttribute("box", S);
-		bb.setAttribute("origin", "http://www.openstreetmap.org/api/0.5");
+		bb.setAttribute("origin", QString("http://www.openstreetmap.org/api/%1").arg(M_PREFS->apiVersion()));
 	}
 
-	std::vector<MapFeature*>::iterator it;
+	QList<MapFeaturePtr>::iterator it;
 	for(it = p->Features.begin(); it != p->Features.end(); it++)
 		(*it)->toXML(o, progress);
 
@@ -532,6 +591,19 @@ ImageMapLayer::ImageMapLayer(const QString & aName, LayerManager* aLayerMgr)
 		setVisible(false);
 	else
 		setVisible(MerkaartorPreferences::instance()->getBgVisible());
+
+#ifdef USE_GDAL
+	if (M_PREFS->getUseShapefileForBackground()) {
+		ImportExportSHP s(NULL);
+		QString fn = WORLD_SHP;
+		if (QDir::isRelativePath(fn))
+			fn = QCoreApplication::applicationDirPath() + "/" + WORLD_SHP;
+		s.loadFile(fn);
+		s.import(this);
+	}
+#endif
+
+	setReadonly(true);
 }
 
 ImageMapLayer::~ ImageMapLayer()
@@ -539,6 +611,18 @@ ImageMapLayer::~ ImageMapLayer()
 	if (layermanager && p->layer_bg)
 		layermanager->removeLayer(p->layer_bg->getLayername());
 	SAFE_DELETE(p->layer_bg);
+	clear();
+}
+
+unsigned int ImageMapLayer::size() const
+{
+#ifdef USE_GDAL
+	//return p->Features.size();
+	if (p->bgType == Bg_Shp)
+		return p->Features.size();
+	else
+#endif
+		return 0;
 }
 
 LayerWidget* ImageMapLayer::newWidget(void)
@@ -550,7 +634,7 @@ LayerWidget* ImageMapLayer::newWidget(void)
 
 void ImageMapLayer::updateWidget()
 {
-	//((ImageLayerWidget*) p->theWidget)->initActions();
+    ((ImageLayerWidget*) p->theWidget)->initActions();
 	p->theWidget->update();
 }
 
@@ -560,7 +644,8 @@ void ImageMapLayer::setVisible(bool b)
 	if (p->bgType == Bg_None)
 		p->Visible = false;
 	else
-	p->layer_bg->setVisible(b);
+		if (p->layer_bg)
+			p->layer_bg->setVisible(b);
 	MerkaartorPreferences::instance()->setBgVisible(p->Visible);
 }
 
@@ -572,9 +657,9 @@ Layer* ImageMapLayer::imageLayer()
 void ImageMapLayer::setMapAdapter(ImageBackgroundType typ)
 {
 	MapAdapter* mapadapter_bg;
-	WmsServerList* wsl;
+	WmsServerList wsl;
 	WmsServer ws;
-	TmsServerList* tsl;
+	TmsServerList tsl;
 	TmsServer ts;
 	QString selws, selts;
 	int idx = -1;
@@ -599,7 +684,7 @@ void ImageMapLayer::setMapAdapter(ImageBackgroundType typ)
 		case Bg_Wms:
 			wsl = MerkaartorPreferences::instance()->getWmsServers();
 			selws = MerkaartorPreferences::instance()->getSelectedWmsServer();
-			ws = wsl->value(selws);
+			ws = wsl.value(selws);
 			wmsa = new WMSMapAdapter(ws.WmsAdress, ws.WmsPath, ws.WmsLayers, ws.WmsProjections,
 					ws.WmsStyles, ws.WmsImgFormat, 256);
 			mapadapter_bg = wmsa;
@@ -611,7 +696,7 @@ void ImageMapLayer::setMapAdapter(ImageBackgroundType typ)
 		case Bg_Tms:
 			tsl = MerkaartorPreferences::instance()->getTmsServers();
 			selts = MerkaartorPreferences::instance()->getSelectedTmsServer();
-			ts = tsl->value(selts);
+			ts = tsl.value(selts);
 			tmsa = new TileMapAdapter(ts.TmsAdress, ts.TmsPath, ts.TmsTileSize, ts.TmsMinZoom, ts.TmsMaxZoom);
 			mapadapter_bg = tmsa;
 			p->layer_bg = new Layer(id(), mapadapter_bg, Layer::MapLayer);
@@ -624,6 +709,16 @@ void ImageMapLayer::setMapAdapter(ImageBackgroundType typ)
 
 			setName("Background - OSM");*/
 			break;
+#ifdef USE_GDAL
+		case Bg_Shp:
+			if (!M_PREFS->getUseShapefileForBackground()) {
+				p->bgType = Bg_None;
+				setName(tr("Map - None"));
+				p->Visible = false;
+			} else
+				setName(tr("Map - Shape"));
+			break;
+#endif
 #ifdef YAHOO
 		case Bg_Yahoo:
 			mapadapter_bg = new YahooLegalMapAdapter();
@@ -750,6 +845,7 @@ TrackMapLayer::TrackMapLayer(const QString & aName, const QString& filename)
 	: MapLayer(aName), Filename(filename)
 {
 	p->Visible = true;
+	p->Readonly = M_PREFS->getReadonlyTracksDefault();
 }
 
 TrackMapLayer::~ TrackMapLayer()
@@ -786,7 +882,8 @@ void TrackMapLayer::extractLayer()
 			PL.clear();
 
 			P = new TrackPoint( S->getNode(0)->position() );
-			P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
+			if (M_PREFS->apiVersionNum() < 0.6)
+				P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
 			P->setTime(S->getNode(0)->time());
 			P->setElevation(S->getNode(0)->elevation());
 			P->setSpeed(S->getNode(0)->speed());
@@ -795,7 +892,8 @@ void TrackMapLayer::extractLayer()
 			unsigned int startP = 0;
 
 			P = new TrackPoint( S->getNode(1)->position() );
-			P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
+			if (M_PREFS->apiVersionNum() < 0.6)
+				P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
 			P->setTime(S->getNode(1)->time());
 			P->setElevation(S->getNode(1)->elevation());
 			P->setSpeed(S->getNode(1)->speed());
@@ -805,7 +903,8 @@ void TrackMapLayer::extractLayer()
 
 			for (unsigned int j=2; j < S->size(); j++) {
 				P = new TrackPoint( S->getNode(j)->position() );
-				P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
+				if (M_PREFS->apiVersionNum() < 0.6)
+					P->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
 				P->setTime(S->getNode(j)->time());
 				P->setElevation(S->getNode(j)->elevation());
 				P->setSpeed(S->getNode(j)->speed());
@@ -828,7 +927,8 @@ void TrackMapLayer::extractLayer()
 
 			Road* R = new Road();
 			R->setLastUpdated(MapFeature::OSMServer);
-			R->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
+			if (M_PREFS->apiVersionNum() < 0.6)
+				R->setTag("created_by", QString("Merkaartor %1").arg(VERSION));
 			theList->add(new AddFeatureCommand(extL,R,true));
 			for (int i=0; i < PL.size(); i++) {
 				theList->add(new AddFeatureCommand(extL,PL[i],true));
@@ -870,13 +970,13 @@ bool TrackMapLayer::toXML(QDomElement xParent, QProgressDialog & progress)
 	o.setAttribute("xmlns", "http://www.topografix.com/GPX/1/1");
 	o.setAttribute("xmlns:rmc", "urn:net:trekbuddy:1.0:nmea:rmc");
 
-	QVector<TrackPoint*>	waypoints;
-	QVector<TrackSegment*>	segments;
-	std::vector<MapFeature*>::iterator it;
+	QList<TrackPoint*>	waypoints;
+	QList<TrackSegment*>	segments;
+	QList<MapFeaturePtr>::iterator it;
 	for(it = p->Features.begin(); it != p->Features.end(); it++) {
-		if (TrackSegment* S = dynamic_cast<TrackSegment*>(*it))
+		if (TrackSegment* S = qobject_cast<TrackSegment*>(*it))
 			segments.push_back(S);
-		if (TrackPoint* P = dynamic_cast<TrackPoint*>(*it))
+		if (TrackPoint* P = qobject_cast<TrackPoint*>(*it))
 			if (!P->tagValue("_waypoint_","").isEmpty())
 				waypoints.push_back(P);
 	}
@@ -951,8 +1051,8 @@ DirtyMapLayer::~ DirtyMapLayer()
 
 DirtyMapLayer* DirtyMapLayer::fromXML(MapDocument* d, const QDomElement e, QProgressDialog & progress)
 {
-	DrawingMapLayer::doFromXML(d->getDirtyLayer(), d, e, progress);
-	return d->getDirtyLayer();
+	DrawingMapLayer::doFromXML(dynamic_cast<DrawingMapLayer*>(d->getDirtyOrOriginLayer()), d, e, progress);
+	return dynamic_cast<DirtyMapLayer*>(d->getDirtyOrOriginLayer());
 }
 
 LayerWidget* DirtyMapLayer::newWidget(void)
@@ -1029,10 +1129,16 @@ DeletedMapLayer::~ DeletedMapLayer()
 {
 }
 
+bool DeletedMapLayer::toXML(QDomElement , QProgressDialog & )
+{
+	return true;
+}
+
 DeletedMapLayer* DeletedMapLayer::fromXML(MapDocument* d, const QDomElement e, QProgressDialog & progress)
 {
-	DrawingMapLayer::doFromXML(d->getTrashLayer(), d, e, progress);
-	return d->getTrashLayer();
+	/* Only keep DeletedMapLayer for backward compatibility with MDC */
+	DrawingMapLayer::doFromXML(dynamic_cast<DrawingMapLayer*>(d->getDirtyOrOriginLayer()), d, e, progress);
+	return NULL;
 }
 
 LayerWidget* DeletedMapLayer::newWidget(void)
@@ -1056,7 +1162,15 @@ OsbMapLayer::OsbMapLayer(const QString & aName)
 	p->Visible = true;
 	pp = new OsbMapLayerPrivate();
 	pp->theImp = new ImportExportOsmBin(NULL);
-	if (pp->theImp->loadFile(aName))
+}
+
+OsbMapLayer::OsbMapLayer(const QString & aName, const QString & filename)
+	: MapLayer(aName)
+{
+	p->Visible = true;
+	pp = new OsbMapLayerPrivate();
+	pp->theImp = new ImportExportOsmBin(NULL);
+	if (pp->theImp->loadFile(filename))
 		pp->theImp->import(this);
 }
 
@@ -1102,26 +1216,26 @@ void OsbMapLayer::invalidate(MapDocument* d, CoordBox vp)
 		for (int j=y1; j <= y2; ++j)
 			tileToLoad.push_back(j*NUM_TILES+i);
 
-	int span = (x2 - x1 + 1) * (y2 - y1 + 1);
-	if (span > TILETOREGION_THRESHOLD)
+	//int span = (x2 - x1 + 1) * (y2 - y1 + 1);
+        if (intToAng(vp.lonDiff()) > M_PREFS->getTileToRegionThreshold())
 		tileToLoad.clear();
 
 	int j;
 	j = 0;
-	while (j<pp->loadedRegions.size()) {
-		if (!regionToLoad.contains(pp->loadedRegions[j])) {
-			if (pp->theImp->clearRegion(pp->loadedRegions[j], d, this))
-				pp->loadedRegions.removeAt(j);
+	while (j<pp->loadedTiles.size()) {
+		if (!tileToLoad.contains(pp->loadedTiles[j])) {
+			if (pp->theImp->clearTile(pp->loadedTiles[j], d, this))
+				pp->loadedTiles.removeAt(j);
 			else 
 				++j;
 		} else
 			++j;
 	}
 	j = 0;
-	while (j<pp->loadedTiles.size()) {
-		if (!tileToLoad.contains(pp->loadedTiles[j])) {
-			if (pp->theImp->clearTile(pp->loadedTiles[j], d, this))
-				pp->loadedTiles.removeAt(j);
+	while (j<pp->loadedRegions.size()) {
+		if (!regionToLoad.contains(pp->loadedRegions[j])) {
+			if (pp->theImp->clearRegion(pp->loadedRegions[j], d, this))
+				pp->loadedRegions.removeAt(j);
 			else 
 				++j;
 		} else
@@ -1145,10 +1259,10 @@ void OsbMapLayer::invalidate(MapDocument* d, CoordBox vp)
 	}
 }
 
-MapFeature*  OsbMapLayer::getFeatureByRef(MapDocument* d, quint64 ref)
-{
-	return pp->theImp->getFeature(d, this, ref);
-}
+//MapFeature*  OsbMapLayer::getFeatureByRef(MapDocument* d, quint64 ref)
+//{
+//	return pp->theImp->getFeature(d, this, ref);
+//}
 
 void OsbMapLayer::setVisible(bool b)
 {
