@@ -24,12 +24,13 @@ class RoadPrivate
 			MetaUpToDate(true)
 		{
 		}
-		std::vector<TrackPoint*> Nodes;
+		std::vector<TrackPointPtr> Nodes;
  		std::vector<Coord> Smoothed;
  		bool SmoothedUpToDate;
 		CoordBox BBox;
 		bool BBoxUpToDate;
 
+		bool IsCoastline;
 		double Area;
 		double Distance;
 		double Width;
@@ -83,7 +84,8 @@ Road::Road(const Road& other)
 Road::~Road(void)
 {
 	for (unsigned int i=0; i<p->Nodes.size(); ++i)
-		p->Nodes[i]->unsetParent(this);
+		if (p->Nodes[i])
+			p->Nodes[i]->unsetParent(this);
 	delete p;
 }
 
@@ -91,10 +93,12 @@ void Road::setLayer(MapLayer* L)
 {
 	if (L)
 		for (unsigned int i=0; i<p->Nodes.size(); ++i)
-			p->Nodes[i]->setParent(this);
+			if (p->Nodes[i])
+				p->Nodes[i]->setParent(this);
 	else
 		for (unsigned int i=0; i<p->Nodes.size(); ++i)
-			p->Nodes[i]->unsetParent(this);
+			if (p->Nodes[i])
+				p->Nodes[i]->unsetParent(this);
 	MapFeature::setLayer(L);
 }
 
@@ -116,9 +120,10 @@ QString Road::description() const
 
 RenderPriority Road::renderPriority(double aPixelPerM) const
 {
-	FeaturePainter* thePainter = getEditPainter(aPixelPerM);
-	if (thePainter && thePainter->isFilled())
-		return RenderPriority(RenderPriority::IsArea,fabs(area()));
+	Q_UNUSED(aPixelPerM)
+	double a = area();
+	if (a)
+		return RenderPriority(RenderPriority::IsArea,fabs(a));
 	return RenderPriority(RenderPriority::IsLinear,0);
 }
 
@@ -151,9 +156,11 @@ unsigned int Road::find(MapFeature* Pt) const
 
 void Road::remove(unsigned int idx)
 {
-	TrackPoint* Pt = p->Nodes[idx];
+	if (p->Nodes[idx]) {
+		TrackPoint* Pt = p->Nodes[idx];
+		Pt->unsetParent(this);
+	}
 	p->Nodes.erase(p->Nodes.begin()+idx);
-	Pt->unsetParent(this);
 	p->BBoxUpToDate = false;
 	p->MetaUpToDate = false;
 	p->SmoothedUpToDate = false;
@@ -191,12 +198,17 @@ const MapFeature* Road::get(unsigned int idx) const
 	return p->Nodes[idx];
 }
 
+bool Road::isNull() const
+{
+	return (p->Nodes.size() == 0);
+}
+
 bool Road::notEverythingDownloaded() const
 {
 	if (lastUpdated() == MapFeature::NotYetDownloaded)
 		return true;
 	for (unsigned int i=0; i<p->Nodes.size(); ++i)
-		if (p->Nodes[i]->notEverythingDownloaded())
+		if (p->Nodes[i] && p->Nodes[i]->notEverythingDownloaded())
 			return true;
 	return false;
 }
@@ -222,6 +234,7 @@ void Road::updateMeta() const
 {
 	p->Area = 0;
 	p->Distance = 0;
+	p->IsCoastline = false;
 
 	if (p->Nodes.size() == 0)
 	{
@@ -233,16 +246,23 @@ void Road::updateMeta() const
 
 	for (unsigned int i=0; (i+1)<p->Nodes.size(); ++i)
 	{
-		const Coord & here = p->Nodes[i]->position();
-		const Coord & next = p->Nodes[i+1]->position();
+		if (p->Nodes[i] && p->Nodes[i+1]) {
+			const Coord & here = p->Nodes[i]->position();
+			const Coord & next = p->Nodes[i+1]->position();
 
-		if (isArea)
-			p->Area += here.lat() * next.lon() - next.lat() * here.lon();
-
-		p->Distance += next.distanceFrom(here);
+			p->Distance += next.distanceFrom(here);
+			//if (isArea)
+				//p->Area += here.lat() * next.lon() - next.lat() * here.lon();
+		}
 	}
 
-	p->Area /= 2;
+	if (isArea)
+		p->Area = p->Distance;
+	//p->Area /= 2;
+
+	if (tagValue("natural","") == "coastline")
+		p->IsCoastline = true;
+
 	p->MetaUpToDate = true;
 }
 
@@ -254,6 +274,19 @@ double Road::distance() const
 	return p->Distance;
 }
 
+bool Road::isCoastline() const
+{
+	if (p->MetaUpToDate == false)
+		updateMeta();
+
+	return p->IsCoastline;
+}
+
+bool Road::isClosed() const
+{
+	return (p->Area > 0.0);
+}
+
 double Road::area() const
 {
 	if (p->MetaUpToDate == false)
@@ -266,7 +299,7 @@ void Road::draw(QPainter& /* thePainter */, const Projection& )
 {
 }
 
-void Road::drawHover(QPainter& thePainter, const Projection& theProjection)
+void Road::drawHover(QPainter& thePainter, const Projection& theProjection, bool solid)
 {
 	// FIXME Selected route
 	if (!size())
@@ -278,20 +311,32 @@ void Road::drawHover(QPainter& thePainter, const Projection& theProjection)
 	F.setWeight(QFont::Black);
 	thePainter.setFont(F);
 	QPen TP(MerkaartorPreferences::instance()->getHoverColor());
-	TP.setWidth(5);
+	TP.setWidth(MerkaartorPreferences::instance()->getHoverWidth());
+	if (!solid) {
+		TP.setDashPattern(getParentDashes());
+	}
 	thePainter.setPen(TP);
+	thePainter.setBrush(Qt::NoBrush);
 	QRegion clipRg = QRegion(thePainter.clipRegion().boundingRect().adjusted(-20, -20, 20, 20));
 	buildPath(theProjection, clipRg);
 	thePainter.drawPath(p->thePath);
-	TP.setWidth(15);
-	TP.setCapStyle(Qt::RoundCap);
-	thePainter.setPen(TP);
-	QPolygonF Pl;
-	buildPolygonFromRoad(this,theProjection,Pl);
-	thePainter.drawPoints(Pl);
+	if (solid) {
+		TP.setWidth(MerkaartorPreferences::instance()->getHoverWidth()*3);
+		TP.setCapStyle(Qt::RoundCap);
+		thePainter.setPen(TP);
+		QPolygonF Pl;
+		buildPolygonFromRoad(this,theProjection,Pl);
+		thePainter.drawPoints(Pl);
+
+		if (M_PREFS->getShowParents()) {
+			for (unsigned int i=0; i<sizeParents(); ++i)
+				if (!getParent(i)->isDeleted())
+					getParent(i)->drawHover(thePainter, theProjection, false);
+		}
+	}
 }
 
-void Road::drawFocus(QPainter& thePainter, const Projection& theProjection)
+void Road::drawFocus(QPainter& thePainter, const Projection& theProjection, bool solid)
 {
 	// FIXME Selected route
 	if (!size())
@@ -303,17 +348,29 @@ void Road::drawFocus(QPainter& thePainter, const Projection& theProjection)
 	F.setWeight(QFont::Black);
 	thePainter.setFont(F);
 	QPen TP(MerkaartorPreferences::instance()->getFocusColor());
-	TP.setWidth(5);
+	TP.setWidth(MerkaartorPreferences::instance()->getFocusWidth());
+	if (!solid) {
+		TP.setDashPattern(getParentDashes());
+	}
 	thePainter.setPen(TP);
+	thePainter.setBrush(Qt::NoBrush);
 	QRegion clipRg = QRegion(thePainter.clipRegion().boundingRect().adjusted(-20, -20, 20, 20));
 	buildPath(theProjection, clipRg);
 	thePainter.drawPath(p->thePath);
-	TP.setWidth(15);
-	TP.setCapStyle(Qt::RoundCap);
-	thePainter.setPen(TP);
-	QPolygonF Pl;
-	buildPolygonFromRoad(this,theProjection,Pl);
-	thePainter.drawPoints(Pl);
+	if (solid) {
+		TP.setWidth(MerkaartorPreferences::instance()->getFocusWidth()*3);
+		TP.setCapStyle(Qt::RoundCap);
+		thePainter.setPen(TP);
+		QPolygonF Pl;
+		buildPolygonFromRoad(this,theProjection,Pl);
+		thePainter.drawPoints(Pl);
+
+		if (M_PREFS->getShowParents()) {
+			for (unsigned int i=0; i<sizeParents(); ++i)
+				if (!getParent(i)->isDeleted())
+					getParent(i)->drawFocus(thePainter, theProjection, false);
+		}
+	}
 }
 
 double Road::pixelDistance(const QPointF& Target, double ClearEndDistance, const Projection& theProjection) const
@@ -321,9 +378,11 @@ double Road::pixelDistance(const QPointF& Target, double ClearEndDistance, const
 	double Best = 1000000;
 	for (unsigned int i=0; i<p->Nodes.size(); ++i)
 	{
-		double x = ::distance(Target,theProjection.project(p->Nodes[i]->position()));
-		if (x<ClearEndDistance)
-			return Best;
+		if (p->Nodes[i]) {
+			double x = ::distance(Target,theProjection.project(p->Nodes[i]));
+			if (x<ClearEndDistance)
+				return Best;
+		}
 	}
 	if (smoothed().size())
 		for (unsigned int i=3; i <p->Smoothed.size(); i += 3)
@@ -340,10 +399,12 @@ double Road::pixelDistance(const QPointF& Target, double ClearEndDistance, const
 	else
 		for (unsigned int i=1; i<p->Nodes.size(); ++i)
 		{
-			LineF F(theProjection.project(p->Nodes[i-1]->position()),theProjection.project(p->Nodes[i]->position()));
-			double D = F.capDistance(Target);
-			if (D < ClearEndDistance)
-				Best = D;
+			if (p->Nodes[i] && p->Nodes[i-1]) {
+				LineF F(theProjection.project(p->Nodes[i-1]),theProjection.project(p->Nodes[i]));
+				double D = F.capDistance(Target);
+				if (D < ClearEndDistance)
+					Best = D;
+			}
 		}
 	return Best;
 }
@@ -361,18 +422,24 @@ void Road::cascadedRemoveIfUsing(MapDocument* theDocument, MapFeature* aFeature,
 					Alternatives.push_back(Pt);
 			}
 			if ( (p->Nodes.size() == 1) && (Alternatives.size() == 0) ) {
-				if (!(layer() == theDocument->getTrashLayer()))
+				if (!isDeleted())
 					theList->add(new RemoveFeatureCommand(theDocument,this));
 			}
 			else
 			{
 				for (unsigned int j=0; j<Alternatives.size(); ++j)
+				{
 					if (i < p->Nodes.size())
+					{
 						if (p->Nodes[i+j] != Alternatives[j])
+						{
 							if ((i+j) == 0)
 								theList->add(new RoadAddTrackPointCommand(this, Alternatives[j], i+j,theDocument->getDirtyOrOriginLayer(layer())));
 							else if (p->Nodes[i+j-1] != Alternatives[j])
 								theList->add(new RoadAddTrackPointCommand(this, Alternatives[j], i+j,theDocument->getDirtyOrOriginLayer(layer())));
+						}
+					}
+				}
 				theList->add(new RoadRemoveTrackPointCommand(this, (TrackPoint*)aFeature,theDocument->getDirtyOrOriginLayer(layer())));
 			}
 		}
@@ -385,63 +452,6 @@ QPainterPath Road::getPath()
 	return p->thePath;
 }
 
-bool QRectInterstects(const QRect& r, const QLine& l, QPoint& a, QPoint& b)
-{
-	QLineF lF = QLineF(l);
-	QPointF pF;
-	bool hasP1 = false;
-	bool hasP2 = false;
-
-	if (QLineF(r.topLeft(), r.bottomLeft()).intersect(lF, &pF) == QLineF::BoundedIntersection) {
-		a = pF.toPoint();
-		hasP1 = true;
-	}
-	if (QLineF(r.bottomLeft(), r.bottomRight()).intersect(lF, &pF) == QLineF::BoundedIntersection) {
-		if (hasP1) {
-			b = pF.toPoint();
-			hasP2 = true;
-		} else {
-			a = pF.toPoint();
-			hasP1 = true;
-		}
-	}
-	if (QLineF(r.bottomRight(), r.topRight()).intersect(lF, &pF) == QLineF::BoundedIntersection) {
-		if (hasP1) {
-			b = pF.toPoint();
-			hasP2 = true;
-		} else {
-			a = pF.toPoint();
-			hasP1 = true;
-		}
-	}
-	if (QLineF(r.topRight(), r.topLeft()).intersect(lF, &pF) == QLineF::BoundedIntersection) {
-		if (hasP1) {
-			b = pF.toPoint();
-			hasP2 = true;
-		} else {
-			a = pF.toPoint();
-			hasP1 = true;
-		}
-	}
-
-	if (hasP1 && hasP2) {
-#if (QT_VERSION >= QT_VERSION_CHECK(4, 4, 0))
-		double la1 = QLineF(a,b).angleTo(lF);
-#else
-		double la1 = QLineF(a,b).angle(lF);
-#endif
-		if (la1 > 15.0 && la1 < 345.0) {
-			QPoint t = b;
-			b = a;
-			a = t;
-		}
-	}
-	if (hasP1)
-		return true;
-	else
-		return false;
-}
-
 void Road::buildPath(Projection const &theProjection, const QRegion& paintRegion)
 {
 	p->thePath = QPainterPath();
@@ -449,21 +459,25 @@ void Road::buildPath(Projection const &theProjection, const QRegion& paintRegion
 		return;
 
 	bool lastPointVisible = true;
-	QPoint lastPoint = theProjection.project(p->Nodes[0]->position());
+	QPoint lastPoint = theProjection.project(p->Nodes[0]);
 	QPoint aP = lastPoint;
 
 	double PixelPerM = theProjection.pixelPerM();
 	double WW = PixelPerM*widthOf()*10+10;
 	QRect clipRect = paintRegion.boundingRect().adjusted(int(-WW-20), int(-WW-20), int(WW+20), int(WW+20));
 
-	if (!clipRect.contains(aP)) {
-		aP.setX(qMax(clipRect.left(), aP.x()));
-		aP.setX(qMin(clipRect.right(), aP.x()));
-		aP.setY(qMax(clipRect.top(), aP.y()));
-		aP.setY(qMin(clipRect.bottom(), aP.y()));
-		lastPointVisible = false;
+
+	if (M_PREFS->getDrawingHack()) {
+		if (!clipRect.contains(aP)) {
+			aP.setX(qMax(clipRect.left(), aP.x()));
+			aP.setX(qMin(clipRect.right(), aP.x()));
+			aP.setY(qMax(clipRect.top(), aP.y()));
+			aP.setY(qMin(clipRect.bottom(), aP.y()));
+			lastPointVisible = false;
+		}
 	}
 	p->thePath.moveTo(aP);
+	QPoint firstPoint = aP;
 	if (smoothed().size())
 	{
 		for (unsigned int i=3; i<smoothed().size(); i+=3)
@@ -474,39 +488,44 @@ void Road::buildPath(Projection const &theProjection, const QRegion& paintRegion
 	}
 	else
 		for (unsigned int j=1; j<size(); ++j) {
-			aP = theProjection.project(p->Nodes[j]->position());
-			if (!clipRect.contains(aP)) {
-				if (!lastPointVisible) {
-					QPoint a, b;
-					if (QRectInterstects(clipRect, QLine(lastPoint, aP), a, b)) {
-						p->thePath.lineTo(a);
-						lastPoint = aP;
-						aP = b;
+			aP = theProjection.project(p->Nodes[j]);
+			if (M_PREFS->getDrawingHack()) {
+				QLine l(lastPoint, aP);
+				if (!clipRect.contains(aP)) {
+					if (!lastPointVisible) {
+						QPoint a, b;
+						if (QRectInterstects(clipRect, l, a, b)) {
+							p->thePath.lineTo(a);
+							lastPoint = aP;
+							aP = b;
+						} else {
+							lastPoint = aP;
+							aP.setX(qMax(clipRect.left(), aP.x()));
+							aP.setX(qMin(clipRect.right(), aP.x()));
+							aP.setY(qMax(clipRect.top(), aP.y()));
+							aP.setY(qMin(clipRect.bottom(), aP.y()));
+						}
 					} else {
+						QPoint a, b;
+						QRectInterstects(clipRect, l, a, b);
 						lastPoint = aP;
-						aP.setX(qMax(clipRect.left(), aP.x()));
-						aP.setX(qMin(clipRect.right(), aP.x()));
-						aP.setY(qMax(clipRect.top(), aP.y()));
-						aP.setY(qMin(clipRect.bottom(), aP.y()));
+						aP = a;
 					}
+					lastPointVisible = false;
 				} else {
-					QPoint a, b;
-					QRectInterstects(clipRect, QLine(lastPoint, aP), a, b);
+					if (!lastPointVisible) {
+						QPoint a, b;
+						QRectInterstects(clipRect, l, a, b);
+						p->thePath.lineTo(a);
+					}
 					lastPoint = aP;
-					aP = a;
+					lastPointVisible = true;
 				}
-				lastPointVisible = false;
-			} else {
-				if (!lastPointVisible) {
-					QPoint a, b;
-					QRectInterstects(clipRect, QLine(lastPoint, aP), a, b);
-					p->thePath.lineTo(a);
-				}
-				lastPoint = aP;
-				lastPointVisible = true;
 			}
 			p->thePath.lineTo(aP);
 		}
+		if (area() > 0.0 && !lastPointVisible)
+			p->thePath.lineTo(firstPoint);
 }
 
 
@@ -537,7 +556,7 @@ bool Road::deleteChildren(MapDocument* theDocument, CommandList* theList)
 			}
 			QList<MapFeature*> ToDeleteKeys = ToDelete.uniqueKeys();
 			for (int i=0; i<ToDeleteKeys.size(); ++i) {
-				if (!theDocument->getTrashLayer()->exists(ToDeleteKeys[i]))
+				if (!ToDeleteKeys[i]->isDeleted())
 					theList->add(new RemoveFeatureCommand(theDocument, ToDeleteKeys[i], Alternatives));
 			}
 			return true;
@@ -580,36 +599,45 @@ double Road::widthOf()
 void Road::setTag(const QString& key, const QString& value, bool addToTagList)
 {
 	MapFeature::setTag(key, value, addToTagList);
+	p->MetaUpToDate = false;
 	p->Width = 0;
 }
 
 void Road::setTag(unsigned int index, const QString& key, const QString& value, bool addToTagList)
 {
 	MapFeature::setTag(index, key, value, addToTagList);
+	p->MetaUpToDate = false;
 	p->Width = 0;
 }
 
 void Road::clearTags()
 {
 	MapFeature::clearTags();
+	p->MetaUpToDate = false;
 	p->Width = 0;
 }
 
 void Road::clearTag(const QString& k)
 {
 	MapFeature::clearTag(k);
+	p->MetaUpToDate = false;
 	p->Width = 0;
 }
 
 QString Road::toXML(unsigned int lvl, QProgressDialog * progress)
 {
+	if (!size()) return "";
+
 	if (progress)
 		progress->setValue(progress->value()+1);
 
 	QString S(lvl*2, ' ');
 	S += QString("<way id=\"%1\">\n").arg(stripToOSMId(id()));
-	for (unsigned int i=0; i<size(); ++i)
-		S += QString((lvl+1)*2, ' ') + QString("<nd ref=\"%1\"/>\n").arg(stripToOSMId(get(i)->id()));
+
+	S += QString((lvl+1)*2, ' ') + QString("<nd ref=\"%1\"/>\n").arg(stripToOSMId(get(0)->id()));
+	for (unsigned int i=1; i<size(); ++i)
+		if (get(i)->id() != get(i-1)->id())
+			S += QString((lvl+1)*2, ' ') + QString("<nd ref=\"%1\"/>\n").arg(stripToOSMId(get(i)->id()));
 	S += tagsToXML(lvl+1);
 	S += QString(lvl*2, ' ') + "</way>\n";
 	return S;
@@ -626,12 +654,23 @@ bool Road::toXML(QDomElement xParent, QProgressDialog & progress)
 	e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
 	e.setAttribute("user", user());
 	e.setAttribute("actor", (int)lastUpdated());
+	e.setAttribute("version", versionNumber());
+	if (isDeleted())
+		e.setAttribute("deleted","true");
 
-	for (unsigned int i=0; i<size(); ++i) {
+	if (size()) {
 		QDomElement n = xParent.ownerDocument().createElement("nd");
 		e.appendChild(n);
+		n.setAttribute("ref", get(0)->xmlId());
 
-		n.setAttribute("ref", get(i)->xmlId());
+		for (unsigned int i=1; i<size(); ++i) {
+			if (get(i)->xmlId() != get(i-1)->xmlId()) {
+				QDomElement n = xParent.ownerDocument().createElement("nd");
+				e.appendChild(n);
+
+				n.setAttribute("ref", get(i)->xmlId());
+			}
+		}
 	}
 
 	tagsToXML(e);
@@ -647,11 +686,13 @@ Road * Road::fromXML(MapDocument* d, MapLayer * L, const QDomElement e)
 		id = "way_" + id;
 	QDateTime time = QDateTime::fromString(e.attribute("timestamp").left(19), "yyyy-MM-ddTHH:mm:ss");
 	QString user = e.attribute("user");
+	bool Deleted = (e.attribute("deleted") == "true");
+	int Version = e.attribute("version").toInt();
 	MapFeature::ActorType A;
 	if (e.hasAttribute("actor"))
 		A = (MapFeature::ActorType)(e.attribute("actor", "2").toInt());
 	else
-		if ((L = d->getDirtyLayer()))
+		if ((L = d->getDirtyOrOriginLayer()))
 			A = MapFeature::User;
 		else
 			A = MapFeature::OSMServer;
@@ -673,6 +714,8 @@ Road * Road::fromXML(MapDocument* d, MapLayer * L, const QDomElement e)
 	}
 	R->setTime(time);
 	R->setUser(user);
+	R->setDeleted(Deleted);
+	R->setVersionNumber(Version);
 
 	QDomElement c = e.firstChildElement();
 	while(!c.isNull()) {
@@ -761,11 +804,6 @@ unsigned int findSnapPointIndex(const Road* R, Coord& P)
 	return BestIdx;
 }
 
-bool Road::isClosed()
-{
-	return size() && (get(0) == get(size()-1));
-}
-
 const std::vector<Coord>& Road::smoothed() const
 {
 	if (!p->SmoothedUpToDate)
@@ -817,6 +855,12 @@ Road* Road::fromBinary(MapDocument* d, OsbMapLayer* L, QDataStream& ds, qint8 c,
 
 	ds >> fSize;
 
+	if (!L) {
+		for (int i=0; i < fSize; ++i)
+			ds >> refId;
+		return NULL;
+	}
+
 	if (id < 1)
 		strId = QString::number(id);
 	else
@@ -848,7 +892,8 @@ Road* Road::fromBinary(MapDocument* d, OsbMapLayer* L, QDataStream& ds, qint8 c,
 		//	sRefId = "node_" + QString::number(refId);
 //		TrackPoint* N = dynamic_cast<TrackPoint*> (L->getFeature(d, refId)); QString("node_"+id));
 //		TrackPoint* N = dynamic_cast<TrackPoint*> (d->getFeature(sRefId));
-		TrackPoint* N = dynamic_cast<TrackPoint*> (L->get(QString("node_%1").arg(refId)));
+//		TrackPoint* N = dynamic_cast<TrackPoint*> (L->get(QString("node_%1").arg(refId)));
+		TrackPoint* N = dynamic_cast<TrackPoint*> (d->getFeature(QString("node_%1").arg(refId)));
 		Q_ASSERT(N);
 		R->add(N);
 	}
@@ -880,15 +925,49 @@ Road * Road::GetSingleParentRoad(MapFeature * mapFeature)
 	for (i=0; i<parents; i++)
 	{
 		MapFeature * parent = mapFeature->getParent(i);
-		Road * road = dynamic_cast<Road*>(parent);
+		Road * road = CAST_WAY(parent);
 
 		if (road == NULL)
 			continue;
 
-		if (parentRoad)
+		if (parentRoad && road != parentRoad)
 			return NULL;
 
-		if (road->layer()->isEnabled())
+		//FIXME This test shouldn't be necessary, but there is at least a case where the road has a NULL layer. The root cause must be found.
+		if (!(road->isDeleted()) && road->layer() && road->layer()->isEnabled())
+			parentRoad = road;
+	}
+
+	return parentRoad;
+}
+
+Road * Road::GetSingleParentRoadInner(MapFeature * mapFeature)
+{
+	unsigned int parents = mapFeature->sizeParents();
+
+	if (parents == 0)
+		return NULL;
+
+	Road * parentRoad = NULL;
+	TrackPoint* trackPoint = dynamic_cast<TrackPoint*>(mapFeature);
+
+	unsigned int i;
+	for (i=0; i<parents; i++)
+	{
+		MapFeature * parent = mapFeature->getParent(i);
+		Road * road = dynamic_cast<Road*>(parent);
+
+		if (road == NULL)
+			continue;
+
+		if (road->isExtrimity(trackPoint) && !road->isClosed())
+			continue;
+		
+		if (parentRoad && road != parentRoad)
+			return NULL;
+
+		//FIXME This test shouldn't be necessary, but there is at least a case where the road has a NULL layer. The root cause must be found.
+		if (!(road->isDeleted()) && road->layer() && road->layer()->isEnabled())
 			parentRoad = road;
 	}
 
