@@ -10,7 +10,8 @@
 #include <QtCore/QString>
 #include <QtGui/QPainter>
 #include <QtGui/QPainterPath>
-
+#include <QMatrix>
+#include <QDomElement>
 #include <math.h>
 
 FeaturePainter::FeaturePainter()
@@ -24,7 +25,7 @@ FeaturePainter::FeaturePainter()
   ForegroundFill(false), DrawTrafficDirectionMarks(false),
   DrawIcon(false), IconScale(0), IconOffset(0),
   DrawLabel(false), LabelScale(0), LabelOffset(0),
-  DrawLabelBackground(false)
+  DrawLabelBackground(false), LabelHalo(false), LabelArea(false)
 {
 }
 
@@ -45,7 +46,7 @@ FeaturePainter::FeaturePainter(const FeaturePainter& f)
   DrawIcon(f.DrawIcon), IconName(f.IconName), IconScale(f.IconScale), IconOffset(f.IconOffset),
   DrawLabel(f.DrawLabel), LabelTag(f.LabelTag), LabelColor(f.LabelColor), LabelScale(f.LabelScale), LabelOffset(f.LabelOffset),
   DrawLabelBackground(f.DrawLabelBackground), LabelBackgroundColor(f.LabelBackgroundColor), LabelBackgroundTag(f.LabelBackgroundTag),
-  LabelFont(f.LabelFont)
+  LabelFont(f.LabelFont), LabelHalo(f.LabelHalo), LabelArea(f.LabelArea)
 {
 	if (f.theSelector)
 		theSelector = f.theSelector->copy();
@@ -97,6 +98,8 @@ FeaturePainter& FeaturePainter::operator=(const FeaturePainter& f)
 	LabelFont = f.LabelFont;
 	LabelTag = f.LabelTag;
 	LabelBackgroundTag = f.LabelBackgroundTag;
+	LabelHalo = f.LabelHalo;
+	LabelArea = f.LabelArea;
 	return *this;
 }
 
@@ -118,6 +121,12 @@ QString asXML(const QColor& c)
 	return "#"+paddedHexa(c.red())+paddedHexa(c.green())+paddedHexa(c.blue())+paddedHexa(c.alpha());
 }
 
+QString colorAsXML(const QString& name, const QColor& c)
+{
+	return
+		name+"Color=\""+asXML(c)+"\"\n";
+}
+
 QString boundaryAsXML(const QString& name, const QColor& c, double Scale, double Offset)
 {
 	return
@@ -130,7 +139,18 @@ QString iconAsXML(const QString& name, const QString& fn, double Scale, double O
 		name+"=\""+fn+"\" "+name+"Scale=\""+QString::number(Scale)+"\" "+name+"Offset=\""+QString::number(Offset)+"\"\n";
 }
 
-QString FeaturePainter::asXML() const
+QColor toColor(const QString& s)
+{
+	return
+		QColor(
+			s.mid(1,2).toInt(0,16),
+			s.mid(3,2).toInt(0,16),
+			s.mid(5,2).toInt(0,16),
+			s.mid(7,2).toInt(0,16));
+}
+
+
+QString FeaturePainter::toXML(QString filename) const
 {
 	QString r;
 	r += "<painter\n";
@@ -149,7 +169,15 @@ QString FeaturePainter::asXML() const
 	if (ForegroundFill)
 		r += " fillColor=\""+::asXML(ForegroundFillFillColor)+"\"\n";
 	if (!IconName.isEmpty() && DrawIcon)
-		r += " " + iconAsXML("icon",IconName, IconScale, IconOffset);
+	{
+		QString iconFilename;
+        if (!IconName.startsWith(':')) {
+            iconFilename = QFileInfo(filename).absoluteDir().relativeFilePath(QFileInfo(IconName).absoluteFilePath());
+        } else {
+            iconFilename = IconName;
+        }
+		r += " " + iconAsXML("icon",iconFilename, IconScale, IconOffset);
+	}
 	if (DrawTrafficDirectionMarks)
 		r += " drawTrafficDirectionMarks=\"yes\"";
 	else
@@ -158,6 +186,10 @@ QString FeaturePainter::asXML() const
 		r += " " + boundaryAsXML("label",LabelColor, LabelScale, LabelOffset);
 		r += " labelFont=\"" + LabelFont.toString() + "\"";
 		r += " labelTag=\"" + LabelTag + "\"";
+		if (LabelHalo)
+			r += " labelHalo=\"yes\"";
+		if (LabelArea)
+			r += " labelArea=\"yes\"";
 	}
 	if (DrawLabelBackground) {
 		r += " labelBackgroundColor=\""+::asXML(LabelBackgroundColor)+"\"";
@@ -172,6 +204,102 @@ QString FeaturePainter::asXML() const
 	return r;
 }
 
+FeaturePainter FeaturePainter::fromXML(const QDomElement& e, QString filename)
+{
+	FeaturePainter FP;
+
+	if (e.hasAttribute("zoomUnder") || e.hasAttribute("zoomUpper"))
+		FP.zoomBoundary(e.attribute("zoomUnder","0").toDouble(),e.attribute("zoomUpper","10e6").toDouble());
+	if (e.hasAttribute("foregroundColor"))
+	{
+		FP.foreground(
+			toColor(e.attribute("foregroundColor")),e.attribute("foregroundScale").toDouble(),e.attribute("foregroundOffset").toDouble());
+		if (e.hasAttribute("foregroundDashDown"))
+			FP.foregroundDash(e.attribute("foregroundDashDown").toDouble(),e.attribute("foregroundDashUp").toDouble());
+	}
+	if (e.hasAttribute("backgroundColor"))
+		FP.background(
+			toColor(e.attribute("backgroundColor")),e.attribute("backgroundScale").toDouble(),e.attribute("backgroundOffset").toDouble());
+	if (e.hasAttribute("touchupColor"))
+	{
+		FP.touchup(
+			toColor(e.attribute("touchupColor")),e.attribute("touchupScale").toDouble(),e.attribute("touchupOffset").toDouble());
+		if (e.hasAttribute("touchupDashDown"))
+			FP.touchupDash(e.attribute("touchupDashDown").toDouble(),e.attribute("touchupDashUp").toDouble());
+	}
+	if (e.hasAttribute("fillColor"))
+		FP.foregroundFill(toColor(e.attribute("fillColor")));
+	if (e.hasAttribute("icon"))
+	{
+		QString iconFilename = e.attribute("icon");
+		if (!QFileInfo(iconFilename).isAbsolute())
+			iconFilename = QFileInfo(filename).absolutePath().append("/").append(iconFilename);
+		FP.setIcon(iconFilename,e.attribute("iconScale", "0.0").toDouble(),e.attribute("iconOffset", "0.0").toDouble());
+	}
+	if (e.attribute("drawTrafficDirectionMarks") == "yes")
+		FP.drawTrafficDirectionMarks();
+	if (e.hasAttribute("labelColor"))
+	{
+		FP.label(
+			toColor(e.attribute("labelColor")),e.attribute("labelScale").toDouble(),e.attribute("labelOffset").toDouble());
+		FP.setLabelFont(e.attribute("labelFont"));
+		FP.labelTag(e.attribute("labelTag"));
+		if (e.hasAttribute("labelHalo"))
+			FP.labelHalo((e.attribute("labelHalo") == "yes"));
+		if (e.hasAttribute("labelArea"))
+			FP.labelArea((e.attribute("labelArea") == "yes"));
+		if (e.hasAttribute("labelBackgroundColor"))
+			FP.labelBackground(toColor(e.attribute("labelBackgroundColor")));
+		if (e.hasAttribute("labelBackgroundTag"))
+			FP.labelBackgroundTag(e.attribute("labelBackgroundTag"));
+	}
+	QDomNode n = e.firstChild();
+	std::vector<std::pair<QString,QString> > Pairs;
+	while (!n.isNull())
+	{
+		if (n.isElement())
+		{
+			QDomElement t = n.toElement();
+			if (t.tagName() == "selector")
+			{
+				if (t.attribute("key") != "")
+					Pairs.push_back(std::make_pair(t.attribute("key"),t.attribute("value")));
+				else
+				{
+					FP.setSelector(t.attribute("expr"));
+					return FP;
+				}
+			}
+		}
+		n = n.nextSibling();
+	}
+	if (Pairs.size() == 1)
+		FP.setSelector(new TagSelectorIs(Pairs[0].first,Pairs[0].second));
+	else if (Pairs.size())
+	{
+		bool Same = true;
+		for (unsigned int i=1; i<Pairs.size(); ++i)
+			if (Pairs[0].first != Pairs[i].first)
+				Same = false;
+		if (Same)
+		{
+			std::vector<QString> Options;
+			for (unsigned int i=0; i<Pairs.size(); ++i)
+				Options.push_back(Pairs[i].second);
+			FP.setSelector(new TagSelectorIsOneOf(Pairs[0].first,Options));
+		}
+		else
+		{
+			std::vector<TagSelector*> Options;
+			for (unsigned int i=0; i<Pairs.size(); ++i)
+				Options.push_back(new TagSelectorIs(Pairs[i].first,Pairs[i].second));
+			FP.setSelector(new TagSelectorOr(Options));
+		}
+	}
+
+	return FP;
+}
+
 
 QString FeaturePainter::userName() const
 {
@@ -221,7 +349,7 @@ FeaturePainter& FeaturePainter::fillActive(bool b)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::foregroundFill(const QColor& FillColor)
+FeaturePainter& FeaturePainter::foregroundFill(QColor FillColor)
 {
 	ForegroundFill = true;
 	ForegroundFillFillColor = FillColor;
@@ -234,7 +362,7 @@ FeaturePainter& FeaturePainter::backgroundActive(bool b)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::background(const QColor& Color, double Scale, double Offset)
+FeaturePainter& FeaturePainter::background(QColor Color, double Scale, double Offset)
 {
 	DrawBackground = true;
 	BackgroundColor = Color;
@@ -269,7 +397,7 @@ FeaturePainter& FeaturePainter::touchupDash(double Dash, double White)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::touchup(const QColor& Color, double Scale, double Offset)
+FeaturePainter& FeaturePainter::touchup(QColor Color, double Scale, double Offset)
 {
 	DrawTouchup = true;
 	TouchupColor = Color;
@@ -293,7 +421,7 @@ FeaturePainter& FeaturePainter::foregroundDash(double Dash, double White)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::foreground(const QColor& Color, double Scale, double Offset)
+FeaturePainter& FeaturePainter::foreground(QColor Color, double Scale, double Offset)
 {
 	DrawForeground = true;
 	ForegroundColor = Color;
@@ -314,6 +442,18 @@ FeaturePainter& FeaturePainter::labelActive(bool b)
 	return *this;
 }
 
+FeaturePainter& FeaturePainter::labelHalo(bool b)
+{
+	LabelHalo = b;
+	return *this;
+}
+
+FeaturePainter& FeaturePainter::labelArea(bool b)
+{
+	LabelArea = b;
+	return *this;
+}
+
 FeaturePainter& FeaturePainter::labelTag(const QString& val)
 {
 	LabelTag = val;
@@ -326,7 +466,7 @@ FeaturePainter& FeaturePainter::labelBackgroundTag(const QString& val)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::label(const QColor& Color, double Scale, double Offset)
+FeaturePainter& FeaturePainter::label(QColor Color, double Scale, double Offset)
 {
 	DrawLabel = true;
 	LabelColor = Color;
@@ -343,7 +483,7 @@ FeaturePainter& FeaturePainter::labelBackgroundActive(bool b)
 	return *this;
 }
 
-FeaturePainter& FeaturePainter::labelBackground(const QColor& bgColor)
+FeaturePainter& FeaturePainter::labelBackground(QColor bgColor)
 {
 	DrawLabelBackground = true;
 	LabelBackgroundColor = bgColor;
@@ -373,6 +513,16 @@ QString FeaturePainter::getLabelTag() const
 	return LabelTag;
 }
 
+bool FeaturePainter::getLabelHalo() const
+{
+	return LabelHalo;
+}
+
+bool FeaturePainter::getLabelArea() const
+{
+	return LabelArea;
+}
+
 QString FeaturePainter::getLabelBackgroundTag() const
 {
 	return LabelBackgroundTag;
@@ -500,9 +650,9 @@ bool FeaturePainter::matchesZoom(double PixelPerM) const
 /*
 void buildPathFromRoad(Road *R, const Projection &theProjection, QPainterPath &Path)
 {
-	Path.moveTo(theProjection.project(R->get(0)->position()));
+	Path.moveTo(theProjection.project(R->get(0)));
 	for (unsigned int i=1; i<R->size(); ++i)
-		Path.lineTo(theProjection.project(R->get(i)->position()));
+		Path.lineTo(theProjection.project(R->get(i)));
 }
 
 void buildPathFromRelation(Relation *R, const Projection &theProjection, QPainterPath &Path)
@@ -515,16 +665,39 @@ void buildPathFromRelation(Relation *R, const Projection &theProjection, QPainte
 */
 void FeaturePainter::drawBackground(Road* R, QPainter& thePainter, const Projection& theProjection) const
 {
-	if (!DrawBackground) return;
-	double PixelPerM = theProjection.pixelPerM();
-	double WW = PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
-	if (WW < 0) return;
-	QPen thePen(BackgroundColor,WW);
-	thePen.setCapStyle(Qt::RoundCap);
-	////thePainter.strokePath(R->getPath(),thePen);
-	thePainter.setPen(thePen);
-	thePainter.setBrush(Qt::NoBrush);
+	if (!DrawBackground && !ForegroundFill) return;
+
+	thePainter.setPen(Qt::NoPen);
+	if (DrawBackground)
+	{
+		double PixelPerM = theProjection.pixelPerM();
+		double WW = PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset;
+		if (WW >= 0)
+		{
+			QPen thePen(BackgroundColor,WW);
+			thePen.setCapStyle(Qt::RoundCap);
+			////thePainter.strokePath(R->getPath(),thePen);
+			thePainter.setPen(thePen);
+		}
+	}
+
+	if (ForegroundFill && (R->size() > 2))
+	{
+		thePainter.setBrush(ForegroundFillFillColor);
+	}
+	else
+		thePainter.setBrush(Qt::NoBrush);
+
+	if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+		thePainter.save();
+		thePainter.setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
+	}
 	thePainter.drawPath(R->getPath());
+	if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+		thePainter.restore();
+	}
+	//thePainter.setBrush(Qt::NoBrush);
+	//thePainter.drawPath(R->getPath());
 	//thePainter.setPen(thePen);
 	//QPointF p1, p2;
 	//p1 = QPointF(R->getPath().elementAt(0));
@@ -551,7 +724,7 @@ void FeaturePainter::drawBackground(Relation* R, QPainter& thePainter, const Pro
 
 void FeaturePainter::drawForeground(Road* R, QPainter& thePainter, const Projection& theProjection) const
 {
-	if (!DrawForeground && !ForegroundFill) return;
+	if (!DrawForeground) return;
 
 	double WW = 0.0;
 	if (DrawForeground)
@@ -571,12 +744,21 @@ void FeaturePainter::drawForeground(Road* R, QPainter& thePainter, const Project
 	}
 	else
 		thePainter.setPen(Qt::NoPen);
-	if (ForegroundFill && (R->size() > 2))
-	{
-		thePainter.setBrush(ForegroundFillFillColor);
-	}
-	else
+	//if (ForegroundFill && (R->size() > 2))
+	//{
+	//	thePainter.setBrush(ForegroundFillFillColor);
+	//}
+	//else
 		thePainter.setBrush(Qt::NoBrush);
+
+	//if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+	//	thePainter.save();
+	//	thePainter.setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
+	//}
+	//thePainter.drawPath(R->getPath());
+	//if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+	//	thePainter.restore();
+	//}
 	thePainter.drawPath(R->getPath());
 }
 
@@ -609,22 +791,54 @@ void FeaturePainter::drawForeground(Relation* R, QPainter& thePainter, const Pro
 	}
 	else
 		thePainter.setBrush(Qt::NoBrush);
+
+	if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+		thePainter.save();
+		thePainter.setOpacity(qreal(M_PREFS->getAreaOpacity()) / 100);
+	}
 	thePainter.drawPath(R->getPath());
+	if (M_PREFS->getAreaOpacity() != 100 && ForegroundFill) {
+		thePainter.restore();
+	}
 }
 
 
 void FeaturePainter::drawTouchup(TrackPoint* Pt, QPainter& thePainter, const Projection& theProjection) const
 {
+	bool IconError = false;
 	if (DrawIcon && (IconName != "") )
 	{
 		double PixelPerM = theProjection.pixelPerM();
 		double WW = PixelPerM*IconScale+IconOffset;
 
         QPixmap pm = getPixmapFromFile(IconName,int(WW));
-        QPoint C(theProjection.project(Pt->position()));
-        thePainter.fillRect(QRect(C-QPoint(2,2),QSize(4,4)),QColor(0,0,0,128));
-        thePainter.drawPixmap( int(C.x()-pm.width()/2), int(C.y()-pm.height()/2) , pm);
-    }
+		if (pm.isNull())
+			IconError = true;
+		else {
+			QPoint C(theProjection.project(Pt));
+			// cbro-20090109: Don't draw the dot if there is an icon
+			// thePainter.fillRect(QRect(C-QPoint(2,2),QSize(4,4)),QColor(0,0,0,128));
+			thePainter.drawPixmap( int(C.x()-pm.width()/2), int(C.y()-pm.height()/2) , pm);
+		}
+	}
+	if (!DrawIcon || (IconName == "") || IconError) 
+	{
+		QColor theColor = QColor(0,0,0,128);
+		if (DrawForeground)
+			theColor = ForegroundColor;
+		else 
+			if (DrawBackground)
+				theColor =BackgroundColor;
+
+		QPointF P(theProjection.project(Pt));
+		if (Pt->findKey("_waypoint_") != Pt->tagSize()) {
+			QRectF R(P-QPointF(4,4),QSize(8,8)); 
+			thePainter.fillRect(R,QColor(255,0,0,128)); 
+		}
+		
+		QRectF R(P-QPointF(2,2),QSize(4,4));
+			thePainter.fillRect(R,theColor);
+	}
 }
 
 void FeaturePainter::drawTouchup(Road* R, QPainter& thePainter, const Projection& theProjection) const
@@ -646,38 +860,50 @@ void FeaturePainter::drawTouchup(Road* R, QPainter& thePainter, const Projection
 			thePainter.strokePath(R->getPath(),thePen);
 		}
 	}
-	if (DrawTrafficDirectionMarks)
+	if ( ((DrawTrafficDirectionMarks) && (M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Oneway)) ||  M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Always)
 	{
-		double theWidth = theProjection.pixelPerM()*R->widthOf()-4;
-		if (theWidth > 8)
-			theWidth = 8;
-		double DistFromCenter = 2*(theWidth+4);
-		if (theWidth > 0)
+		MapFeature::TrafficDirectionType TT = trafficDirection(R);
+		if ( (TT != MapFeature::UnknownDirection) || (M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Always) ) 
 		{
-			for (unsigned int i=1; i<R->size(); ++i)
+			double theWidth = theProjection.pixelPerM()*R->widthOf()-4;
+			if (theWidth > 8)
+				theWidth = 8;
+			double DistFromCenter = 2*(theWidth+4);
+			if (theWidth > 0)
 			{
-				QPointF FromF(theProjection.project(R->getNode(i-1)->position()));
-				QPointF ToF(theProjection.project(R->getNode(i)->position()));
-				if (distance(FromF,ToF) > (DistFromCenter*2+4))
+				for (unsigned int i=1; i<R->size(); ++i)
 				{
-					QPointF H(FromF+ToF);
-					H *= 0.5;
-					double A = angle(FromF-ToF);
-					QPointF T(DistFromCenter*cos(A),DistFromCenter*sin(A));
-					QPointF V1(theWidth*cos(A+M_PI/6),theWidth*sin(A+M_PI/6));
-					QPointF V2(theWidth*cos(A-M_PI/6),theWidth*sin(A-M_PI/6));
-					MapFeature::TrafficDirectionType TT = trafficDirection(R);
-					if ( (TT == MapFeature::OtherWay) || (TT == MapFeature::BothWays) )
+					QPointF FromF(theProjection.project(R->getNode(i-1)));
+					QPointF ToF(theProjection.project(R->getNode(i)));
+					if (distance(FromF,ToF) > (DistFromCenter*2+4))
 					{
-						thePainter.setPen(QColor(0,0,0));
-						thePainter.drawLine(H+T,H+T-V1);
-						thePainter.drawLine(H+T,H+T-V2);
-					}
-					if ( (TT == MapFeature::OneWay) || (TT == MapFeature::BothWays) )
-					{
-						thePainter.setPen(QColor(0,0,0));
-						thePainter.drawLine(H-T,H-T+V1);
-						thePainter.drawLine(H-T,H-T+V2);
+						QPointF H(FromF+ToF);
+						H *= 0.5;
+						double A = angle(FromF-ToF);
+						QPointF T(DistFromCenter*cos(A),DistFromCenter*sin(A));
+						QPointF V1(theWidth*cos(A+M_PI/6),theWidth*sin(A+M_PI/6));
+						QPointF V2(theWidth*cos(A-M_PI/6),theWidth*sin(A-M_PI/6));
+						if ( M_PREFS->getDirectionalArrowsVisible() == DirectionalArrows_Oneway )
+						{
+							if ( (TT == MapFeature::OtherWay) || (TT == MapFeature::BothWays) )
+							{
+								thePainter.setPen(QColor(0,0,0));
+								thePainter.drawLine(H+T,H+T-V1);
+								thePainter.drawLine(H+T,H+T-V2);
+							}
+							if ( (TT == MapFeature::OneWay) || (TT == MapFeature::BothWays) )
+							{
+								thePainter.setPen(QColor(0,0,0));
+								thePainter.drawLine(H-T,H-T+V1);
+								thePainter.drawLine(H-T,H-T+V2);
+							}
+						} 
+						else
+						{
+							thePainter.setPen(QColor(255,0,0));
+							thePainter.drawLine(H-T,H-T+V1);
+							thePainter.drawLine(H-T,H-T+V2);
+						}
 					}
 				}
 			}
@@ -690,16 +916,8 @@ void FeaturePainter::drawTouchup(Road* R, QPainter& thePainter, const Projection
 #define BG_SPACING 6
 #define BG_PEN_SZ 2
 
-void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Projection& theProjection) const
+void FeaturePainter::drawPointLabel(QPoint C, QString str, QString strBg, QPainter& thePainter, const Projection& theProjection) const
 {
-	if (!DrawLabel)
-		return;
-
-	QString str = Pt->tagValue(getLabelTag(), "");
-	QString strBg = Pt->tagValue(getLabelBackgroundTag(), "");
-	if (str.isEmpty() && strBg.isEmpty())
-		return;
-
 	LineParameters lp = labelBoundary();
 	double PixelPerM = theProjection.pixelPerM();
 	double WW = PixelPerM*lp.Proportional+lp.Fixed;
@@ -713,7 +931,6 @@ void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Proje
 	int modY = 0;
 	QPainterPath textPath;
 	QPainterPath bgPath;
-	QPoint C(theProjection.project(Pt->position()));
 
 	if (!str.isEmpty()) {
 		modX = - (metrics.width(str)/2);
@@ -730,8 +947,6 @@ void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Proje
 		textPath.addText(modX, modY, font, str);
 		thePainter.translate(C);
 
-		thePainter.setPen(QPen(Qt::white, WW/5));
-		thePainter.drawPath(textPath);
 	}
 	if (DrawLabelBackground && !strBg.isEmpty()) {
 		modX = - (metrics.width(strBg)/2);
@@ -752,6 +967,10 @@ void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Proje
 		thePainter.setBrush(LabelBackgroundColor);
 		thePainter.drawPath(bgPath);
 	}
+	if (getLabelHalo()) {
+		thePainter.setPen(QPen(Qt::white, font.pixelSize()/5));
+		thePainter.drawPath(textPath);
+	}
 	thePainter.setPen(Qt::NoPen);
 	thePainter.setBrush(LabelColor);
 	thePainter.drawPath(textPath);
@@ -766,6 +985,21 @@ void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Proje
 }
 
 
+void FeaturePainter::drawLabel(TrackPoint* Pt, QPainter& thePainter, const Projection& theProjection) const
+{
+	if (!DrawLabel)
+		return;
+
+	QString str = Pt->tagValue(getLabelTag(), "");
+	QString strBg = Pt->tagValue(getLabelBackgroundTag(), "");
+
+	if (str.isEmpty() && strBg.isEmpty())
+		return;
+
+	QPoint C(theProjection.project(Pt));
+	drawPointLabel(C, str, strBg, thePainter, theProjection);
+}
+
 void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection& theProjection) const
 {
 	if (!DrawLabel)
@@ -776,12 +1010,19 @@ void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection&
 	if (str.isEmpty() && strBg.isEmpty())
 		return;
 
+	if (getLabelArea()) {
+		QPoint C(theProjection.project(R->boundingBox().center()));
+		drawPointLabel(C, str, strBg, thePainter, theProjection);
+		return;
+	}
+
 	LineParameters lp = labelBoundary();
 	double PixelPerM = theProjection.pixelPerM();
 	double WW = PixelPerM*R->widthOf()*lp.Proportional+lp.Fixed;
 	if (WW < 10) return;
 	//double WWR = qMax(PixelPerM*R->widthOf()*BackgroundScale+BackgroundOffset, PixelPerM*R->widthOf()*ForegroundScale+ForegroundOffset);
 
+ 	QPainterPath textPath;
     QFont font = getLabelFont();
 
 	if (!str.isEmpty()) {
@@ -789,13 +1030,14 @@ void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection&
 		font.setPixelSize(int(WW));
 		QFontMetrics metrics(font);
 
-		if (font.pixelSize() >= 5) {
-			thePainter.setPen(LabelColor);
+		if (font.pixelSize() >= 5 && R->getPath().length() > metrics.width(str)) {
 			thePainter.setFont(font);
-			int repeat = int((R->getPath().length() / (metrics.width(str) * LABEL_PATH_DISTANCE)) - 0.5);
+
+			int repeat = int((R->getPath().length() / ((metrics.width(str) * LABEL_PATH_DISTANCE))) - 0.5);
 			int numSegment = repeat+1;
 			qreal lenSegment = R->getPath().length() / numSegment;
 			qreal startSegment = 0;
+		 	QPainterPath textPath;
 			do {
 				QRegion rg = thePainter.clipRegion();
 
@@ -812,25 +1054,31 @@ void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection&
 					qreal t = R->getPath().percentAtLength(curLen);
 					QPointF pt = R->getPath().pointAtPercent(t);
 					qreal angle = R->getPath().angleAtPercent(t);
-					QString txt;
-					txt.append(str[i]);
-					thePainter.save();
-					thePainter.translate(pt);
-					//qDebug()<<"txt = "<<txt<<", angle = "<<angle<<curLen<<t;
-					thePainter.rotate(-angle+modAngle);
-					//p->thePainter.drawText(QRect(0, modY, metrics.width(txt), WW), Qt::AlignVCenter, txt);
-					modY = (metrics.ascent()/2)-2;
-					thePainter.drawText(0, modY, txt);
-					thePainter.restore();
+					modY = (metrics.ascent()/2)-3;
 
-					//rg -= QRect(0, modY, metrics.width(txt), metrics.height()).translated(pt.toPoint());
+					QMatrix m;
+					m.translate(pt.x(), pt.y());
+					m.rotate(-angle+modAngle);
 
-					qreal incremenet = metrics.width(txt);
+					QPainterPath charPath;
+					charPath.addText(0, modY, font, str.mid(i, 1));
+					charPath = charPath * m;
+					
+					textPath.addPath(charPath);
+
+					qreal incremenet = metrics.width(str[i]);
 					curLen += (incremenet * modIncrement);
 				}
 				startSegment += lenSegment;
 			} while (--repeat >= 0);
 
+			if (getLabelHalo()) {
+				thePainter.setPen(QPen(Qt::white, font.pixelSize()/6));
+				thePainter.drawPath(textPath);
+			}
+			thePainter.setPen(Qt::NoPen);
+			thePainter.setBrush(LabelColor);
+			thePainter.drawPath(textPath);
 			thePainter.setClipRegion(rg);
 		}
 	}
@@ -875,6 +1123,10 @@ void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection&
 				thePainter.setBrush(LabelBackgroundColor);
 				thePainter.drawPath(bgPath);
 
+				if (getLabelHalo()) {
+					thePainter.setPen(QPen(Qt::white, font.pixelSize()/5));
+					thePainter.drawPath(textPath);
+				}
 				thePainter.setPen(Qt::NoPen);
 				thePainter.setBrush(LabelColor);
 				thePainter.drawPath(textPath);
@@ -891,6 +1143,79 @@ void FeaturePainter::drawLabel(Road* R, QPainter& thePainter, const Projection&
 	}
 }
 
+/* GlobalPainter */
+
+GlobalPainter::GlobalPainter()
+: DrawBackground(false)
+{
+}
+
+GlobalPainter::GlobalPainter(const GlobalPainter& f)
+: DrawBackground(f.DrawBackground), BackgroundColor(f.BackgroundColor)
+{
+}
+
+GlobalPainter& GlobalPainter::operator=(const GlobalPainter& f)
+{
+	if (&f == this) return *this;
+
+	DrawBackground = f.DrawBackground;
+	BackgroundColor = f.BackgroundColor;
+
+	return *this;
+}
+
+GlobalPainter::~GlobalPainter()
+{
+}
+
+QString GlobalPainter::toXML() const
+{
+	QString r;
+	r += "<global\n";
+	if (DrawBackground)
+		r += " " + colorAsXML("background",BackgroundColor);
+	r += "/>\n";
+	return r;
+}
+
+GlobalPainter GlobalPainter::fromXML(const QDomElement& e)
+{
+	GlobalPainter FP;
+
+	if (e.hasAttribute("backgroundColor")) {
+		FP.backgroundActive(true);
+		FP.background(toColor(e.attribute("backgroundColor")));
+	}
+
+	return FP;
+}
+
+GlobalPainter& GlobalPainter::backgroundActive(bool b)
+{
+	DrawBackground = b;
+	return *this;
+}
+
+bool GlobalPainter::getDrawBackground() const
+{
+	return DrawBackground;
+}
+
+GlobalPainter& GlobalPainter::background(QColor Color)
+{
+	DrawBackground = true;
+	BackgroundColor = Color;
+	return *this;
+}
+
+QColor GlobalPainter::getBackgroundColor() const
+{
+	return BackgroundColor;
+}
+
+/* */
+
 PaintStyleLayer::~PaintStyleLayer()
 {
 }
@@ -910,3 +1235,5 @@ PaintStyleLayer* PaintStyle::get(unsigned int i)
 	return Layers[i];
 }
 
+
+
