@@ -37,6 +37,7 @@ public slots:
 	void on_cbMapAdapter_currentIndexChanged(int index);
 	void on_buttonBox_clicked(QAbstractButton * button);
 	void on_BrowseStyle_clicked();
+	void on_BrowseTemplate_clicked();
 	void on_btAdapterSetup_clicked();
 	void on_btBgColor_clicked();
 	void on_btFocusColor_clicked();
@@ -54,9 +55,13 @@ public slots:
 	void on_btBrowse_clicked();
 
 private:
+	void updateStyles();
 	void loadPrefs();
+	void initLanguages(QComboBox*);
 	void savePrefs();
 
+	void changeEvent(QEvent *);
+
 private:
 	QVector<Tool> theTools;
 	QColor BgColor;
