@@ -3,6 +3,7 @@
 #include "MainWindow.h"
 #include "Command/Command.h"
 #include "Map/MapDocument.h"
+#include "Map/MapLayer.h"
 #include "Map/DownloadOSM.h"
 #include "Sync/DirtyList.h"
 
@@ -25,6 +26,9 @@ void syncOSM(MainWindow* theMain, const QString& aWeb, const QString& aUser, con
 		DirtyListExecutor Exec(theMain->document(),Future,aWeb,aUser,aPwd, UseProxy, ProxyHost, ProxyPort,Describer.tasks());
 		Exec.executeChanges(theMain);
 
+		if (M_PREFS->getAutoHistoryCleanup() && !theMain->document()->getDirtyOrOriginLayer()->getDirtySize())
+			theMain->document()->history().cleanup();
+
 		if (theMain->fileName != "") {
 			if (MerkaartorPreferences::instance()->getAutoSaveDoc()) {
 				theMain->saveDocument();
