@@ -9,13 +9,14 @@
 class MDockAncestor : public QDockWidget
 {
 	public:
-		MDockAncestor(QWidget *parent = 0) : QDockWidget(parent) {};
-		void setWidget ( QWidget * widget ) { QDockWidget::setWidget(widget); };
+        MDockAncestor(QWidget *parent = 0) : QDockWidget(parent) {}
+        void setWidget ( QWidget * widget ) { QDockWidget::setWidget(widget); }
 		QWidget* getWidget();
-		void setAllowedAreas ( Qt::DockWidgetAreas areas )  { QDockWidget::setAllowedAreas(areas); };
+        void setAllowedAreas ( Qt::DockWidgetAreas areas )  { QDockWidget::setAllowedAreas(areas); }
 
 	protected:
 		QWidget* mainWidget;
+		virtual void retranslateUi() = 0;
 };
 
 #else
@@ -35,6 +36,7 @@ class MDockAncestor : public QDialog
 	protected:
 		QVBoxLayout* theLayout;
 		QWidget* mainWidget;
+		virtual void retranslateUi() = 0;
 };
 
 #endif
